## Script (Python) "pa_innerlink_upload"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=field_name,  new_url, new_title='', new_description='', widget_index=None
##title=
##

return context.pa_innercontent_upload(
    field_name=field_name,
    widget_index=widget_index, 
    max_size_property=None, 
    max_size_field_name=None,
    title=new_title, 
    description=new_description, 
    attachedLink=new_url)


# -*- coding: utf-8 -*-
## Test case for PloneArticleTool
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for all InnerContentProxy classes
"""

__docformat__ = 'restructuredtext'

# CMF imports
from Products.CMFCore.utils import getToolByName

# Products imports
from Products.PloneArticle.config import PLONEARTICLE_TOOL
from Products.PloneArticle.tests.common import BaseTestCase

expected_ref_types = {
    'PloneArticle': {
        'files': ['File'],
        'images': ['Image', 'News Item'],
        'links': ['Image', 'Topic', 'Large Plone Folder', 'Document', \
            'PloneArticleMultiPage', 'Favorite', 'Event', 'Folder', 'Link', \
            'News Item', 'File', 'PloneArticle']
        }
    }

update_types = {
    'PloneArticle': {
        'images': ['Image', 'Link'],
        'links': ['Link',]
        }
    }

updated_ref_types = {
    'PloneArticle': {
        'files': ['File'],
        'images': ['Image',],
        'links': ['Link',]
        }
    }

class FakeImage:
    def __init__(self):
        self.width = 200
        self.height = 160

class ArticleToolTestCase(BaseTestCase):
    """
    Test PloneArticleTool
    """

    def afterSetUp(self):
        self.tool = getToolByName(self.portal, PLONEARTICLE_TOOL)

    def testUpdateReferenceableTypes(self):
        tool = self.tool
        self.failUnless(hasattr(tool, 'referenceable_types'))
        
        tool.updateReferenceableTypes()               
        ref_types = tool.referenceable_types
        for schema_id in ('files', 'images', 'links'):
            value = ref_types['PloneArticle'][schema_id]
            value.sort()
            expected_value = expected_ref_types['PloneArticle'][schema_id]
            expected_value.sort()
            self.assertEqual(value, expected_value)

        tool.updateReferenceableTypes(updates=update_types)
        ref_types = tool.referenceable_types
        self.assertEqual(ref_types, updated_ref_types)
        self.failIf('Link' in ref_types['PloneArticle']['images'])

        del tool.referenceable_types['PloneArticle']['files']
        tool.updateReferenceableTypes()   
        ref_types = tool.referenceable_types
        value = ref_types['PloneArticle']['files']
        value.sort()
        expected_value = expected_ref_types['PloneArticle']['files']
        expected_value.sort()
        self.assertEqual(value, expected_value)

        tool.updateReferenceableTypes(reset=True)
        ref_types = tool.referenceable_types
        for schema_id in ('files', 'images', 'links'):
            value = ref_types['PloneArticle'][schema_id]
            value.sort()
            expected_value = expected_ref_types['PloneArticle'][schema_id]
            expected_value.sort()
            self.assertEqual(value, expected_value)

        tool.updateReferenceableTypes(updates=update_types, reset=True)
        ref_types = tool.referenceable_types
        for schema_id in ('files', 'images', 'links'):
            value = ref_types['PloneArticle'][schema_id]
            value.sort()
            expected_value = expected_ref_types['PloneArticle'][schema_id]
            expected_value.sort()
            self.assertEqual(value, expected_value)
        
    def testGetScaleSize(self):
        tool = self.tool
        img = FakeImage()
        expected_sizes = (100, 80)
        self.assertEqual(tool._getScaleSize(img, 100, 80), expected_sizes)
        expected_sizes = (100, 80)
        self.assertEqual(tool._getScaleSize(img, 100, 100), expected_sizes)
        
        
    def test_getSetEnabledModelsForType(self):
        tool = self.tool
        models = tool.listModels()
        self.failIf(len(models) == 0, "No models registered, aborting test %s")

        tmpl_ids = [m.id for m in models]
        tmpl_ids.sort()
        tool.setEnabledModelsForType('PloneArticle', tmpl_ids, tmpl_ids[0])
        availables, default = tool.getEnabledModelsForType('PloneArticle')
        availables = list(availables)
        self.assertEquals(availables, tmpl_ids)
        self.assertEquals(default, tmpl_ids[0])

        # setting a default view not listed in views should not raise exception
        exception_raised = True
        try:
            tool.setEnabledModelsForType('PloneArticle', tmpl_ids, 'dummy__')
            exception_raised = False
        except ValueError:
            pass
        self.failIf(exception_raised,
                    "Exception raised when setting a dummy default template "
                    "not listed in views")

    def test_cleanFilename(self):
        files = (
            ('C:\something\with spaces\windows name.txt', 'windows name.txt'),
            ('/the/unix naming/way/unix name.txt', 'unix name.txt')
            )

        for fullpath, expected in files:
            result = self.tool.cleanFilename(fullpath)
            self.assertEquals(result, expected)

    def testMigrationFinished(self):
        """Check version is correctly set after install"""
        t = self.tool
        self.assertEqual(t.getVersion(),
                         t.getVersionFromFS())

    def testNeedsVersionMigration(self):
        """No migration required immediately after install!"""
        t = self.tool
        self.failIf(
            t.needsVersionMigration(),
            'Tool needs upgrading, currently: %s, FS version: %s' % (
                str(t.getVersion()), t.getVersionFromFS()
                )
            )

        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ArticleToolTestCase))
    return suite
            




# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: test_browse_image.py 5783 2007-01-21 17:59:47Z roeder $
import unittest
from funittest import logical
from funittest import dataprovider
from funittest import scripts
from funittest import interpreter
from funittest.testrunner import Test
from funittest import Schema
from funittest import register_tst

import os

class BrowseImage(Test):
    schema = Schema()
    
    _uses_file_upload = 1


    def setUp(self):
        interpreter.open("")        
        user=dataprovider.cmfplone.user.get('sampleadmin')
        logical.cmfplone.application.login(user)        
        article = dataprovider.plonearticle.article.get("article1")
        scripts.plonearticle.article.existing_article(article)

    def step_1(self):
        "Browse an image to be added to the article"
        interpreter.annotate("Test: Browe a image to be added to the article")
        image = dataprovider.plonearticle.articleimage.get("Image 1")
        logical.plonearticle.article.browse_image(image)

    def test(self):
        self.expect_ok(1)

register_tst("PloneArticle", BrowseImage())



# -*- coding: utf-8 -*-
## Test case for PloneArticle Install procedure
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for PloneArticle Install procedure
"""

__docformat__ = 'restructuredtext'

# Zope
from zope.component import getUtility
from zope.app.publisher.interfaces.browser import IBrowserMenu

#CMF
from Products.CMFCore.utils import getToolByName

from Products.PloneTestCase.setup import PLONE25, PLONE30

# Products imports
from Products.PloneArticle.tests.common import BaseTestCase
from Products.PloneArticle.config import PLONEARTICLE_TOOL

class TestInstallation(BaseTestCase):
    """
    """

    def afterSetUp(self):
        self.qi = getToolByName(self.portal, 'portal_quickinstaller')
        self.ttool = getToolByName(self.portal, 'portal_types')
        self.wf = getToolByName(self.portal, 'portal_workflow')
        # As ZopeTestCase REQUEST has no 'PARENTS', PlacelessTranslationService
        # raises a KeyError and cannot translate. This is an (ugly) workaround.
        self.app.REQUEST.set('PARENTS', [self.portal, self.app])
        return

    def test_installedAllTypes(self):
        """ test types installed """
        ttool = self.ttool
        installed_types = tids = ttool.objectIds()

        to_install = ('PloneArticle', 'FileInnerContentProxy',
                      'ImageInnerContentProxy', 'LinkInnerContentProxy',)

        not_to_install = ('BaseInnerContentProxy',)

        for tid in to_install:
            self.failUnless(tid in installed_types,
                            '%s is not installed' % tid,)
            tinfo = ttool[tid]
            self.failUnless(tinfo.product == 'PloneArticle', tinfo.product)

        for tid in not_to_install:
            self.failIf(tid in installed_types, '%s is installed' % tid)

    def testWorkflowChains(self):

        default_chain = ('simple_publication_workflow',)
        mapping = {
            'PloneArticle': default_chain,
            'FileInnerContentProxy': (),
            'ImageInnerContentProxy': (),
            'LinkInnerContentProxy': (),
            'InnerContentContainer': (),
            'PloneArticleTool': (),
            }

        for pt, wf in mapping.items():
            pwf = self.wf.getChainFor(pt)
            self.failUnlessEqual(pwf, wf, (pt, pwf, wf))

    def test_skin_installed(self):
        stool = getToolByName(self.portal, 'portal_skins')
        ids = stool.objectIds()
        self.failUnless('plonearticle' in ids,
                        "'plonearticle' not found in portal_skins")

    def testToolInstalled(self):
        atool = getToolByName(self.portal, PLONEARTICLE_TOOL)
        self.failUnless(atool is not None)

    def testGetAddableTypesInMenu(self):
        self.loginAsPortalOwner()
        addable_type_ids = (
            'PloneArticle',)
        not_addable_type_ids = (
            'FileInnerContentProxy',
            'ImageInnerContentProxy',
            'LinkInnerContentProxy',
            'InnerContentContainer',
            'PloneArticleTool',
            )
        allowed_types = self.portal.getAllowedTypes()
        allowed_type_ids = [x.getId() for x in allowed_types]
        self.failUnless(len([x for x in addable_type_ids if x in allowed_type_ids]) == len(addable_type_ids))
        self.failUnless([x for x in not_addable_type_ids if x not in allowed_type_ids])
        menu_types = getUtility(IBrowserMenu, name='plone_contentmenu_factory', context=self.folder)
        menu_types = menu_types.getMenuItems(self.portal, self.app.REQUEST)
        menu_type_ids = [x['extra']['id'] for x in menu_types]
        self.failUnless(len([x for x in addable_type_ids if x.lower() in menu_type_ids]), len(addable_type_ids))
        self.failUnless([x for x in not_addable_type_ids if x.lower() not in menu_type_ids])
        self.logout()

    def testTypesNotSearched(self):
        test_type_ids = (
            'FileInnerContentProxy',
            'ImageInnerContentProxy',
            'LinkInnerContentProxy',
            'InnerContentContainer',
            'PloneArticleTool',
            )
        ptool = getToolByName(self.portal, 'portal_properties')
        types_not_searched = ptool.site_properties.types_not_searched
        self.failUnless(len([x for x in test_type_ids if x in types_not_searched]) == len(test_type_ids))

    def testTypesNotToList(self):
        test_type_ids = (
            'FileInnerContentProxy',
            'ImageInnerContentProxy',
            'LinkInnerContentProxy',
            'InnerContentContainer',
            'PloneArticleTool',
            )
        ptool = getToolByName(self.portal, 'portal_properties')
        types_not_to_list = ptool.navtree_properties.metaTypesNotToList
        self.failUnless(len([x for x in test_type_ids if x in types_not_to_list]) == len(test_type_ids))

    def testKupuConfiguration(self):

        ktool = getToolByName(self.portal, 'kupu_library_tool')
        self.failUnless('FileInnerContentProxy' in ktool.getPortalTypesForResourceType('linkable'))
        self.failUnless('PloneArticle' in ktool.getPortalTypesForResourceType('collection'))
        self.failUnless('InnerContentContainer' in ktool.getPortalTypesForResourceType('collection'))
        self.failUnless('ImageInnerContentProxy' in ktool.getPortalTypesForResourceType('mediaobject'))

    def testPortalFactoryConfiguration(self):

        ftool = getToolByName(self.portal, 'portal_factory')
        types = ftool.getFactoryTypes()
        self.failUnless('PloneArticle' in types)
        self.failUnless('FileInnerContentProxy' in types)
        self.failUnless('ImageInnerContentProxy' in types)
        self.failUnless('LinkInnerContentProxy' in types)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    return suite



# -*- coding: utf-8 -*-
## Defines BaseInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines BaseInnerContentProxy and BaseFileContentProxy
"""

__docformat__ = 'restructuredtext'

# Zope imports
from zope.interface import implements
from zope.component import queryUtility

import transaction
from AccessControl import ClassSecurityInfo

# CMF
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.atapi import BaseContentMixin
from Products.Archetypes.atapi import MinimalSchema
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import ReferenceField
from Products.Archetypes.atapi import TextField
from Products.Archetypes.config import UUID_ATTR
from Products.Archetypes.utils import shasattr

from plone.i18n.normalizer.interfaces import IUserPreferredURLNormalizer
from plone.i18n.normalizer.interfaces import IURLNormalizer

# Other imports
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import \
    ReferenceBrowserWidget
from Products.Archetypes.Widget import TextAreaWidget

from Products.PloneArticle.config import PLONEARTICLE_TOOL
from Products.PloneArticle.interfaces import IBaseInnerContentProxy

# No need to have all dublin core metadata
BaseInnerContentProxySchema = MinimalSchema.copy() + Schema((
    TextField(
        'description',
        searchable=True,
        default='',
        accessor="Description",
        widget=TextAreaWidget(
            label='Description',
            description="A short summary of the content",
            label_msgid="label_description",
            description_msgid="help_description",
            i18n_domain="plone"),
        ),
    ReferenceField(
        'referencedContent',
        relationship='article_proxy',
        allowed_types_method='getReferenceablePortalTypes',
        keepReferencesOnCopy=True,
        widget=ReferenceBrowserWidget(
            label='Referenced content',
            label_msgid='label_referenced_content',
            i18n_domain='plonearticle',
            ),
        ),
))

class BaseInnerContentProxy(BaseContentMixin):
    """This is a proxy between article and a referenced content.

    A referenced content can be defined outside the article and a proxy is just
    a placeholder for this external object or internal object.

    A proxy is a traversable content so it can be accessed from its url.
    """

    implements(IBaseInnerContentProxy)
    security = ClassSecurityInfo()

    schema = BaseInnerContentProxySchema
    meta_type = 'BaseInnerContentProxy'
    archetype_name = 'Base Content Inner Proxy'
    typeDescription= ''
    typeDescMsgId  = ''
    global_allow = False
    _at_rename_after_creation = True

    # subclasses *must* define this attribute. see getReferencedContentInterfaces
    # referenceable_interfaces = ()

    def __setattr__(self, key, value):
        # Patch this method to get the original uid of source object
        if key == UUID_ATTR and value is None:
            if not shasattr(self, '_v_src_uid'):
                self._v_src_uid = getattr(self, UUID_ATTR, None)

        BaseContentMixin.__setattr__(self, key, value)

    security.declareProtected(CCP.View, 'getAttachedContentField')
    def getAttachedContentField(self):
        """
        Return the field marked has 'attached_content'
        """
        schema = self.Schema()
        fields = schema.filterFields(attached_content=True)
        if len(fields) > 0:
            return fields[0]
        return None

    security.declareProtected(CCP.View, 'getPrimaryValue')
    def getPrimaryValue(self, referenced_fieldname, attached_fieldname,
                        default=None):
        """Returns value of referenced field or attached field.

        Attached field has priority on referenced field.
        @param referenced_fieldname: field we want to check in the referenced
        content.
        @param attached_fieldname: attached field of the inner proxy
        @param default: Defautl value if referenced and attached field are empty
        """

        # If an attached field is defined return its value
        field = self.getField(attached_fieldname)
        if field.get_size(self) > 0:
            accessor = field.getAccessor(self)
            return accessor()

        # Attached field is empty. Use referenced content
        referenced_content = self.getReferencedContent()
        if referenced_content is not None:
            field = referenced_content.getField(referenced_fieldname)
            if field is not None and field.get_size(referenced_content) > 0:
                accessor = field.getAccessor(referenced_content)
                return accessor()

        return default

    def getReferenceableInterfaces(klass,):
        """
        Returns tuple of interfaces that a content class must provide so that
        the proxy can reference it. Usually the proxy will also provide this
        interface.

        A proxy class must have an attribute 'referenced_content_interfaces'
        that is a tuple of interfaces.
        """
        return klass.referenceable_interfaces
    getReferenceableInterfaces = classmethod(getReferenceableInterfaces)

    security.declareProtected(CCP.View, 'getReferenceablePortalTypes')
    def getReferenceablePortalTypes(self, field):
        """
        Returns a tuple of portal types that this proxy can reference, for
        the particular portal_type using this proxy (i.e, PloneArticle.images
        types can be different from PloneArticleLike.images)
        """
        pat = getToolByName(self, PLONEARTICLE_TOOL)
        return pat.getReferenceablePortalTypesFor(self, field)

    security.declareProtected(CCP.View, 'SearchableText')
    def SearchableText(self):
        """
        Returns string used by SearchableText index.
        Use BaseObject SearchableText method and add referencedContent
        SearchableText value
        """

        # Get default SearchableText
        value = BaseContentMixin.SearchableText(self)

        # Get referenced content
        content = self.getReferencedContent()

        if content is None:
            return value

        # Add referenced content Searchable text
        try:
            value += ' ' + content.SearchableText()
        except AttributeError:
            pass
        return value


def innerContentAdded(ob, event):
    """Supersedes the BaseInnerContentProxy.manage_afterAdd with events machinery,
    see events.zcml"""

    item = event.object
    container = event.newParent
    # FIXME: Uuuuugly, but required by Archetypes that has no events machinery
    super(ob.__class__, ob).manage_afterAdd(item, container)

    # Get source object UID
    is_copy = getattr(item, '_v_is_cp', None)
    src_uid = getattr(ob, '_v_src_uid', None)

    if is_copy and src_uid is not None:
        # Get source object
        atool = getToolByName(ob, "archetype_tool")
        src_obj = atool.getObject(src_uid)

        if src_obj is not None:
            # Get references from source object
            refs = src_obj.getReferenceImpl()

            # Copy references from source object to new created object
            for ref in refs:
                ob.addReference(ref.getTargetObject(), relationship=ref.relationship)

    return


class BaseFileContentProxy(BaseInnerContentProxy):
    """
    Rename content id according to attached file name
    """
    security = ClassSecurityInfo()
    _at_rename_after_creation = False


    security.declarePrivate('renameFromFileName')
    def renameFromFileName(self, field=None):
        """
        field must be a FileField or derivative. If field is None, we use
        getAttachedContentField.
        """
        if field is None:
            field = self.getAttachedContentField()

        filename = field.getFilename(self)

        if not filename:
            return

        if not isinstance(filename, unicode):
            charset = self.getCharset()
            filename = unicode(filename, charset)

        request = getattr(self, 'REQUEST', None)
        if request is not None:
            normalize = IUserPreferredURLNormalizer(request).normalize
        else:
            normalize = queryUtility(IURLNormalizer).normalize

        clean_filename = normalize(filename)

        self_id = self.getId()
        if clean_filename != self_id:
            # new id to be set
            invalid_id = True
            check_id = getattr(self, 'check_id', None)

            if check_id is not None:
                invalid_id = check_id(clean_filename, required=1)

            if invalid_id:
                parent_ids = self.getParentNode().objectIds()
                base_name = clean_filename
                extension = ''
                dot_pos = clean_filename.rfind('.')
                if dot_pos > 0:
                    base = clean_filename[:dot_pos]
                    extension = clean_filename[dot_pos:]

                idx = 0
                while clean_filename in parent_ids \
                          and clean_filename != self_id:
                    idx += 1
                    clean_filename = "%s-%d%s" % (base, idx, extension)

            if clean_filename == self_id:
                # we have recomputed the same numbered file name. Do nothing
                return

            # got a clean file name - rename it
            # apply subtransaction. w/o a subtransaction renaming
            # fails when the type is created using portal_factory
            transaction.savepoint(optimistic=True)
            self.setId(clean_filename)
            self.reindexObject()




# -*- coding: utf-8 -*-
## Defines interfaces inherited from IBaseInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines interfaces inherited from IBaseInnerContentField
"""

__docformat__ = 'restructuredtext'

# Zope imports
from zope.interface import Interface

class IBaseInnerContentField(Interface):
    """"""
    
    pass

class IFileInnerContentField(IBaseInnerContentField):
    """"""
    
    pass

class IImageInnerContentField(IBaseInnerContentField):
    """"""
    
    pass
    
class ILinkInnerContentField(IBaseInnerContentField):
    """"""
    
    pass

class IBaseInnerContent(Interface):
    """"""
    
    pass

class IInnerContentContainer(Interface):
    """"""
    
    pass



# -*- coding: utf-8 -*-
## This module contains all content types used in PloneArticle product
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains all content types used in PloneArticle product
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from zope.interface import implements
from zope import event

# CMF imports
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.event import ObjectInitializedEvent
from Products.Archetypes.event import ObjectEditedEvent
from Products.Archetypes.utils import mapply

try:
    from Products.LinguaPlone.public import registerType, Schema
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import registerType, Schema

# Products imports
from Products.ATContentTypes.content.document import ATDocument, \
     ATDocumentSchema

from Products.PloneArticle.config import PLONEARTICLE_TOOL

from Products.PloneArticle.interfaces import IPloneArticle
from Products.PloneArticle.field import FileInnerContentField, \
     ImageInnerContentField, LinkInnerContentField
from Products.PloneArticle.widget import FileInnerContentWidget, \
    ImageInnerContentWidget, LinkInnerContentWidget
from Products.PloneArticle.content.mixin import ArticleMixin
from Products.PloneArticle.pafti import PloneArticleFactoryTypeInformation

from Products.PloneArticle.version import CURRENT_ARTICLE_VERSION


# Inherits schema from ATDocument
# Copy it to make sure PloneArticle schema is totally independant
PloneArticleSchema = ATDocumentSchema.copy() + Schema((
    FileInnerContentField(
        'files',
        searchable=True,
        schemata='files',
        widget=FileInnerContentWidget(
            label='Files',
            label_msgid='label_files',
            i18n_domain='plonearticle',
            ),
        ),
    ImageInnerContentField(
        'images',
        searchable=True,
        schemata='images',
        widget=ImageInnerContentWidget(
            label='Images',
            label_msgid='label_images',
            i18n_domain='plonearticle',
            ),
        ),
    LinkInnerContentField(
        'links',
        searchable=True,
        schemata='links',
        widget=LinkInnerContentWidget(
            label='Links',
            label_msgid='label_links',
            i18n_domain='plonearticle',
            ),
        ),
    ),)

# We need to change the order of schematas such we got default, files, image, links, others...

PloneArticleSchema.moveField('files', after='text')
PloneArticleSchema.moveField('images', after='files')
PloneArticleSchema.moveField('links', after='images')


_marker = []

class PloneArticle(ArticleMixin, ATDocument):
    """A rich document containing files, images, links"""

    implements(IPloneArticle)

    # FIXME: We need to add this Zope 2 interface because the Archetypes tool
    # doesn't care about Zope 3 interfaces. Feature or bug?
    __implements__ = ATDocument.__implements__

    # Standard content type setup
    schema = PloneArticleSchema
    typeDescMsgId = 'description_edit_plonearticle'

    _at_fti_meta_type = PloneArticleFactoryTypeInformation.meta_type

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    # Enable marshalling via WebDAV/FTP/ExternalEditor.
    __dav_marshall__ = True

    __pa_version = ()

    # Get the standard actions (tabs)
    security = ClassSecurityInfo()

    security.declareProtected(CCP.ModifyPortalContent, 'initializeArchetype')
    def initializeArchetype(self, **kwargs):
        """called by the generated add* factory in types tool

        Overwritten to call the 'right' initializeArchetype and set PA version
        number at creation time.
        """
        ATDocument.initializeArchetype(self, **kwargs)

        self.__pa_version = CURRENT_ARTICLE_VERSION

    security.declareProtected(CCP.ManagePortal, 'getPAVersion')
    def getPAVersion(self):
        """Return the version used to create this article
        """
        return self.__pa_version

    security.declareProtected(CCP.ManagePortal, 'setPAVersion')
    def setPAVersion(self, version):
        """Set the version used to create this article. Usually called after
        migration
        """
        self.__pa_version = version

    ### ISelectableBrowserDefault overrides ###
    def getAvailableLayouts(self):
        """Get the layouts from enabled models (configured by plonarticle_tool).
        """
        tool = getToolByName(self, PLONEARTICLE_TOOL)
        models = tool.getModelRegistry()
        model_ids, default = tool.getEnabledModelsForType(self.aq_explicit.portal_type)
        result = []
        for mid in model_ids:
            model = models.get(mid, None)
            if model is None:
                continue
            result.append((mid, model.Title()))

        return result

    def canSetDefaultPage(self):
        """
        Override BrowserDefaultMixin because default page stuff doesn't make
        sense for articles.
        """
        return False       
    
    security.declarePrivate('_processForm')
    def _processForm(self, data=1, metadata=None, REQUEST=None, values=None):
        """BaseObject._processForm override 
          this patch is done to make processForm possible by fieldset
          when IMultiPageSchema is not implemented
        """
    
        request = REQUEST or self.REQUEST
        if values:
            form = values
        else:
            form = request.form
              
        fieldset = form.get('fieldset', None)
        if fieldset is None :
            ArticleMixin._processForm(self, data=data, metadata=metadata,
                                      REQUEST=REQUEST, values=values)
        else :                  
            schema = self.Schema()
            schemata = self.Schemata()
            fields = []
    
            fields = schemata[fieldset].fields()                
    
            form_keys = form.keys()
    
            for field in fields:
    
                if not field.writeable(self):
    
                    continue
    
                widget = field.widget
                result = widget.process_form(self, field, form,
                                             empty_marker=_marker)
                if result is _marker or result is None: continue
    
                # Set things by calling the mutator
                mutator = field.getMutator(self)
                __traceback_info__ = (self, field, mutator)
                result[1]['field'] = field.__name__
                mapply(mutator, result[0], **result[1])
    
            self.reindexObject()        

registerType(PloneArticle)



# -*- coding: utf-8 -*-
## PloneArticle
##
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

__docformat__ = 'restructuredtext'


# articles have a private version attribute (integer type) starting from 0
#
# Each release including a change in article content type must be appended here
#
# WARNING: NEVER REMOVE ANY ENTRY IN THIS LIST.
#          ALWAYS ADD A NEW VERSION AT THE END (APPEND)
__VERSIONS = [
    '4.0.0-beta4',
    ]

# __VERSIONS[idx] = version_string (i.e, '4.0.0beta4')
# VERSIONS[version_string] = idx
VERSIONS = dict([(__VERSIONS[idx], idx) for idx in range(len(__VERSIONS))])

CURRENT_ARTICLE_VERSION = VERSIONS[__VERSIONS[-1]]

def getArticleVersionFor(version):
    return VERSIONS.get(version, None)

def getArticleVersionStringFor(version_num):
    return __VERSIONS[version_num]

__all__ = ('CURRENT_ARTICLE_VERSION', 'getArticleVersionFor',
           'getArticleVersionStringFor')



# -*- coding: utf-8 -*-
## Product description
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from Products.Archetypes.Registry import Registry, getDoc
from Products.PloneArticle.browser.interface import IPloneArticleModelView

from Products.PloneArticle.i18n import ModelMessageFactory as _

class ModelDescription:

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, klass, title='', description=''):
        if not IPloneArticleModelView.implementedBy(klass):
            raise ValueError, \
                  "%s does not implement IPloneArticleModelView" % repr(klass)
        
        self.id = klass.template_id
        self.klass = klass
        self.title = title or klass.title or klass.__name__
        self.title_msgid = klass.title_msgid
        self.description = description or getDoc(klass)
        self.description_msgid = klass.description_msgid
        self.icon = klass.icon
        
    def Title(self):
        """Return the translated title of this model"""
        return _(self.title_msgid, default=self.title)

    def Description(self):
        """Return the translated description"""
        return _(self.description_msgid, default=self.description)

modelRegistry = Registry(ModelDescription)

def registerArticleModel(klass):
    """
    """
    model = ModelDescription(klass)
    modelRegistry.register(model.id, model)

def getModelRegistry():
    return modelRegistry



# -*- coding: utf-8 -*-
## Defines BaseInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines BaseInnerContentField
$Id: baseinnercontent.py 8575 2008-06-23 15:13:26Z encolpe $
"""

__docformat__ = 'restructuredtext'

# Python imports
from types import ListType, TupleType, DictType
from itertools import izip, count

# Zope imports
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from ZODB.POSException import ReadConflictError
from zope.interface import implements

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Field import Field
from Products.Archetypes.utils import shasattr

# Products imports
from Products.PloneArticle.field import InnerContentContainer
from Products.PloneArticle.interfaces import IBaseInnerContentField, \
    IBaseInnerContentProxy

class BaseInnerContentField(Field):
    """An inner content field contains inner content objects

    Inner content objects are stored in a folderish sub object of instance
    """

    implements(IBaseInnerContentField)

    _properties = Field._properties.copy()
    _properties.update({
        'type' : 'object',
        'inner_portal_type' : 'BaseInnerContent',
        'default': (),
        'default_content_type': None,
        })

    security  = ClassSecurityInfo()

    getContainerName = Field.getName

    security.declarePrivate('get')
    def get(self, instance, **kwargs):
        # If mimetype is text/plain returns indexable value
        mimetype = kwargs.get("mimetype", None)

        if mimetype == "text/plain":
            return self.getIndexableValue(instance)

        # Returns a list of inner content objects
        name = self.getContainerName()
        try:
            container = instance._getOb(name)
        except ReadConflictError:
            raise
        except Exception, msg:
            return ()

        if isinstance(container, InnerContentContainer):
            return container.objectValues()
        return ()

    # LinguaPlone needs this, as it thinks it always deals with ObjectField...
    security.declarePrivate('getRaw')
    def getRaw(self, instance, **kwargs):
        return self.get(instance, **kwargs)

    security.declarePrivate('get_size')
    def get_size(self, instance, **kwargs):
        """Sum of all inner content objects size"""
        objs = self.get(instance, **kwargs)
        size = 0
        for obj in objs:
            size += obj.get_size()
        return size

    security.declarePublic('getContentType')
    def getContentType(self, instance, fromBaseUnit=True):
        """ return MIME type """
        f = self.get(instance)
        return getattr(f, 'content_type', self.default_content_type)

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        """
            Set inner fields of this field.

            self is an instance of BaseInnerContentField instance is the
            instance of the article we are working on.

            value can be:
                InnerContentContainer
                A list of InnerContentProxy objects:
                A list of dictionaries: Each item of the dictionary will be handled

            kwargs are optionnals arguments changing the way this field is
            updated :
                "update": If True, update inner content proxies present in the value's dict list.
                          If False, update inner content proxies present in the value's dict list,
                          but delete all inner content proxies that are missing from the list but
                          have been created before.
                "_initializing_": creates a new inner container

            This method calls the mutator of each inner field of this field, by
            sending to it only what has been returned by the associated widget.

            Example of the value when the image already existed and hasn't changed:

            ({'description': ('Existing image', {}),
              'id': ('imageinnercontentproxy.2007-01-30.0017453092', {}),
              'title': ('Existing image', {})},)

            Example of value when there is a new image:

            ({'description': ('New image', {}),
              'id': ('imageinnercontentproxy.2007-02-04.7516632708', {}),
              'attachedImage': (<BaseUnit at fss.JPG>, {}),
              'title': ('New image', {})})

            ({'description': ('Referenced image', {}),
              'referencedContent': ('817b168ec61fbfcb080de02629431631', {}),
              'id': ('imageinnercontentproxy.2007-02-04.1880613482', {}),
              'title': ('Referenced image', {})})

        """

        if kwargs.get('_initializing_', False):
            return

        if isinstance(value, InnerContentContainer):
            return self.set(instance, value.objectValues(), **kwargs)

        # value must be a list of dictionnary
        # Each dictionnary defines values of inner content fields
        # Values can be wrapped into a tuple if for example you want to
        # define some extra args on the field mutator
        if type(value) not in (ListType, TupleType,):
            raise ValueError, \
                  "Value must be a list or a tuple, get: %s" % type(value)

        # Value can be a list of dictionnary or a list of InnerContent proxy
        # objects. In this case, wrap object into a dictionnary
        for index in range(0, len(value)):
            item = value[index]

            if IBaseInnerContentProxy.providedBy(item):
                schema = item.Schema()
                data = {}

                # Loop on fields and extract data using accessor
                for field in schema.fields():
                    if 'w' not in field.mode:
                        continue

                    field_id = field.getName()
                    accessor = field.getAccessor(item)

                    if accessor is None:
                        continue

                    data[field_id] = accessor()

                value[index] = data

        # Get inner container
        container_name = self.getContainerName()
        if shasattr(instance, container_name):
            container = getattr(instance, container_name, None)
        else:
            container = None

        # Do nothing if value is empty and container doesn't already exist
        if not isinstance(container, InnerContentContainer) and not value:
            return

        # Create a new container if it doesn't already exist
        if not isinstance(container, InnerContentContainer):
            container = InnerContentContainer(container_name)
            instance._setObject(container_name, container)
            container = instance._getOb(container_name)
            container.initializeArchetype()
            #instance._p_changed = 1

        # 'update' argument is a switch for innercontent deletion:
        # - if True, ids that are not present in the request WON'T be destroyed
        # - if False or doesn't exist, all ids that are not in request WILL be deleted.
        update = kwargs.get('update', False)

        # Format all field values to be a tuple. If no extra args defined,
        # replace by an empty dictionnary
        for item in value:
            for item_key in item.keys():
                item_value = item[item_key]

                if type(item_value) not in (TupleType, ListType,):
                    item[item_key] = (item_value, {})

        # Build a list containing all inner content ids to be replaced or updated
        inner_content_ids = [x['id'][0] for x in value]

        # If update not in kwargs, delete objects in container
        if not update and container is not None:
            inner_content_ids_to_remove = [x for x in container.objectIds()
                                           if x not in inner_content_ids]
            container.manage_delObjects(ids=inner_content_ids_to_remove)

        # Check for new inner contents
        for item, item_index in izip(value, count()):
            inner_content_field_values = item.copy()
            inner_content_id = inner_content_field_values['id'][0]
            # no need to reindex until a new inner content is really added
            to_reindex = False
            # Create a new inner content if it doesn't exist
            if not shasattr(container, inner_content_id):
                container.invokeFactory(type_name=self.inner_portal_type,
                                        id=inner_content_id)
                to_reindex = True

            # reorder content only if update not in kwargs
            if not update:
                content_position = container.getObjectPosition(inner_content_id)
                if content_position != item_index:
                    container.moveObjectToPosition(inner_content_id, item_index)

            inner_content = getattr(container, inner_content_id)
            # removing id from field_values since id shouldn't be updated
            # (it's the key)
            del inner_content_field_values['id']

            # Update inner content fields (new and already existing)
            for field_name, field_args in inner_content_field_values.items():
                field = inner_content.getField(field_name)
                mutator = field.getMutator(inner_content)
                mutator(field_args[0], **field_args[1])
                to_reindex = True

            # Then reindex object
            if to_reindex:
                inner_content.reindexObject()

    security.declarePublic('getInnerContentSchema')
    def getInnerContentSchema(self, instance):
        """Returns a inner content schema."""

        atool = getToolByName(instance, 'archetype_tool')
        registered_schemas = [x['schema'] for x in atool.listRegisteredTypes()
                              if x['portal_type'] == self.inner_portal_type]
        schema = registered_schemas[0]
        return schema

    security.declarePublic('getInnerContentAttachedField')
    def getInnerContentAttachedField(self, instance):
        """Return the field instance responsible of holding an attached file, or
        None if no field is defined for this.
        """
        schema = self.getInnerContentSchema(instance)
        fields = schema.filterFields(attached_content=True)
        count = len(fields)
        if count == 0:
            return None
        if count == 1:
            return fields[0]

        raise (AssertionError,
               "%d fields are defined as holder for attached content for %s "
               "field: %s" % (count, instance.portal_type, self.getName()))

    security.declarePublic('getTemporaryInnerContent')
    def getTemporaryInnerContent(self, instance):
        """Returns a temporary inner content"""

        atool = getToolByName(instance, "archetype_tool")
        types = atool.listRegisteredTypes()
        types = [t for t in types if t['portal_type'] == self.inner_portal_type]

        # Inner Content is not registered
        if not types:
            raise ValueError, "Inner content is not registered: %s" % \
                self.inner_portal_type

        # Create temporary inner content
        inner_klass = types[0]['klass']
        inner_content_id = instance.generateUniqueId(self.inner_portal_type)
        inner_content = inner_klass(inner_content_id)
        inner_content._at_is_fake_instance = True
        inner_content._is_fake_instance = True
        wrapped = inner_content.__of__(instance)
        wrapped.initializeArchetype()
        return wrapped

    security.declarePrivate('getIndexableValue')
    def getIndexableValue(self, instance):
        """
        Returns string value used on SearchableText index
        """

        proxies = self.get(instance)
        return " ".join([x.SearchableText() for x in proxies])

    security.declarePublic('writeable')
    def writeable(self, instance, debug=False):
        """Overriding standard Field.writeable such this field is not writeable
        on temporary objects (using portal_factory)"""

        if instance.isTemporary():
            return False
        return super(BaseInnerContentField, self).writeable(instance, debug)


registerField(BaseInnerContentField,
              title='BaseInnerContent',
              description=('Used to store objects implementing IBaseInnerContent.')
             )



# -*- coding: utf-8 -*-
## Product description
##
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Multi-page content type stuff
"""

# Zope imports
from AccessControl import ClassSecurityInfo
from zope.interface import implements

# CMF imports
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Archetypes imports
try:
    from Products.LinguaPlone.public import registerType, Schema
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import registerType, Schema

from Products.Archetypes.Field import BooleanField
from Products.Archetypes.Widget import BooleanWidget

from Products.ATContentTypes.content.folder import ATFolder

from Products.PloneArticle.interfaces import IPloneArticleMultiPage


schema = ATFolder.schema.copy() \
         + Schema((
    BooleanField(
        'viewTOCFirst',
        default=False,
        widget=BooleanWidget(
            label='View table of contents first',
            label_msgid='label_view_toc_first',
            i18n_domain='plonearticle',
            visible={'view' : 'invisible'},
        )
    ),
    BooleanField(
        'viewTOC',
        default=True,
        widget=BooleanWidget(
            label='Display table of contents in dropdown list',
            label_msgid='label_display_toc_dropdown',
            description='If you have checked properties to view TOC first, this property will not be applied.',
            description_msgid='description_display_toc_dropdown',
            i18n_domain='plonearticle',
            visible={'view' : 'invisible'},
        )
    ),
    ))


class PloneArticleMultiPage(ATFolder):
    """ A folder containing articles """

    implements(IPloneArticleMultiPage)

    typeDescMsgId = 'description_edit_multipage'
    schema = schema

    security = ClassSecurityInfo()

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    security.declareProtected(CCP.View, 'getPages')
    def getPages(self, full_objects=False):
        """
        Return the ordered list of articles as brains
        """

        catalog = getToolByName(self, 'portal_catalog')
        mtool = getToolByName(self, 'portal_membership')

        show_inactive = mtool.checkPermission('Access inactive portal content', self)

        path = {'query': '/'.join(self.getPhysicalPath())
                ,'depth': 1
                }

        return catalog(path = path
                       ,sort_on='getObjPositionInParent'
                       ,show_all=1
                       ,show_inactive=show_inactive)



        #return self.getFolderContents(full_objects)

    security.declareProtected(CCP.View, 'getViewTOC')
    def getViewTOC(self):
        """Returns true if you can access TOC in dropdown menu"""

        field = self.getField('viewTOC')
        value = field.get(self)

        if not value and not self.getViewTOCFirst():
            return False

        return True


    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        """
        Always False: the default page is automatic for multipage
        """
        return False

    security.declareProtected(CCP.View, 'getPAMDefaultPage')
    def getPAMDefaultPage(self):
        """
        Return the first user visible article as default page if
        ViewTOC is unchecked (i.e, it redirect the user to the first page)
        """

        request = getattr(self, 'REQUEST', None)
        force_toc = request.get('force_toc', False)
        pages = self.getPages()

        if force_toc or self.getViewTOCFirst() or len(pages) == 0:
            return ATFolder.__call__(self)
        else:
            return request.RESPONSE.redirect(pages[0].getObject().absolute_url())


registerType(PloneArticleMultiPage)



# -*- coding: utf-8 -*-
## Product description
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Explorer view
"""

__docformat__ = 'restructuredtext'

# Zope imports
import zLOG
from zope.interface import Interface, implements
from Products.Five import BrowserView

from Products.CMFCore.utils import getToolByName

from Products.ATContentTypes.interface import IImageContent

from interface import IExplorerView
from Products.PloneArticle.config import PLONEARTICLE_TOOL as TOOL_ID
from Products.PloneArticle.interfaces import IPloneArticle

class ExplorerView(BrowserView):
    """View for article explorer
    """
    implements(IExplorerView)

    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.path = {'query': '/'.join(context.getPhysicalPath()) }
        self.article = request.get('current_article', None)
        if self.article is None:
            raise ValueError, ("%s called without 'current_article' in REQUEST",
                               self.__class__.__name__)
        
        self.field_name = request.get('field_name', None)
        self.scope = request.get('scope', '')
        self.searchTerm = request.get('searchTerm', None)
        if self.field_name is None:
            raise ValueError, (
                "Invalid: %s called without 'field_name' in request"
                % self.__class__.__name__)

        # a proper call is self.queryMethod.aq_inner: if you don't do that,
        # getFolderContents will issue an AttributeError for
        # 'portal_membership', for instance        
        if context.portal_type == 'Topic' or self.searchTerm :
            self.queryMethod = [context.queryCatalog]
            if self.scope=='portal':
               self.path={}
        else:
            self.queryMethod = [context.getFolderContents] 
            self.path['depth']=1
        self.isArticle = IPloneArticle.providedBy(context)

    def getFolders(self):
        if self.isArticle:
            return []
        if self.searchTerm:
            return []    
        return self.queryMethod[0]({'path': self.path,
                                    'is_folderish': True})
    
    def getReferenceableContent(self):
        pa_tool = getToolByName(self.context, TOOL_ID)
        field = self.article.getField(self.field_name)        
        proxy = field.getTemporaryInnerContent(self.article)
        ref_types = proxy.getReferenceablePortalTypes(field)
        results = self.queryMethod[0]({'path': self.path,
                                       'portal_type': ref_types,
                                       'SearchableText': self.searchTerm })
        return results

    def isImage(self, obj):
        return IImageContent.providedBy(obj)


    def getOrientationFor(self, image_obj):        

        field = image_obj.getField('image')
        im_width, im_height = field.getSize(image_obj)
        height = width = 0

        if im_height >= im_width:
            return 'portrait'

        return 'landscape'
                                       
    #FIXME: return an HTML tag is sooooo ugly
    def getThumbNailTagFor(self, image_obj):
        
        title = image_obj.title_or_id()
        pa_tool = getToolByName(self.context, TOOL_ID)

        # compute thumbnail
        field = image_obj.getField('image')
        im_width, im_height = field.getSize(image_obj)
        tag_size = 0

        if im_height >= im_width:
            tag_size = 100
        else:
            tag_size = int(70 * im_width / im_height)    
            
        return pa_tool.getThumbnailTag(image_obj, 'image',
                                       maximizeTo=tag_size,
                                       title='Select %s' % title,
                                       alt='Select %s' % title)                                       



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from funittest import interpreter
from funittest import register_pom

class AttachmentBrowse:
    """
    When the user edits an article, he can access the different attachments,
    image, link and file.
    The user can click on the browse button to add an attachment to the
    article, which already exists somewhere in the site.
    The browse form is the same for the different attachment types, so
    this model is designed to be reusable.
    """
    def access(self):
        """
        Access the browse view by clicking on the browse link.
        """
        interpreter.click("plone-article-browse-link")        
        # XXX Wait for condition!
                
register_pom("PloneArticle", AttachmentBrowse())



# -*- coding: utf-8 -*-
## PloneArticle product provides content with files, images, links
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module contains all mixins used in PloneArticle
"""

# Zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore import permissions as CCP

# Archetypes imports
try:
    from Products.LinguaPlone.public import BaseFolder
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import BaseFolder


class ArticleMixin(BaseFolder):
    """Inherits from this mixin to get some article features"""

    security = ClassSecurityInfo()

    security.declareProtected(CCP.View, 'getArticleObject')
    def getArticleObject(self,):
        """Returns the article object itself.

        It is used by objects implementing IBaseInnerContentProxy.
        """

        return self

InitializeClass(ArticleMixin)



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: test_add_file.py 5781 2007-01-21 17:55:37Z roeder $
import unittest
from funittest import logical
from funittest import dataprovider
from funittest import scripts
from funittest import interpreter
from funittest import scenarios
from funittest.testrunner import Test
from funittest import Schema
from funittest import register_tst

import os

class AddFile(Test):
    schema = Schema()

    _uses_file_upload = 1

    def setUp(self):    
        interpreter.open("")
        user=dataprovider.cmfplone.user.get('sampleadmin')
        logical.cmfplone.application.login(user)
        self._article = dataprovider.plonearticle.article.get("article1")
        scenarios.cmfplone.addcontent(content=self._article)

    def step_1(self):
        "Add a file to an article"
        interpreter.annotate("Test: Add a file to the article")
        file = dataprovider.plonearticle.articlefile.get("File 1")
        logical.plonearticle.article.add_file(file)
        logical.plonearticle.article.save_article(self._article)

    def test(self):
        self.expect_ok(1)

register_tst("PloneArticle", AddFile())



## Script (Python) "pa_thumb"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Returns a cacheable thumbnail
##

from Products.CMFCore.utils import getToolByName
NotFound = "NotFound"
request = context.REQUEST
response = request.RESPONSE

if len(traverse_subpath) < 1:
    raise NotFound, "Unknown page."

thumbnail_id = traverse_subpath[0]
abtool = getToolByName(context, 'plonearticle_tool')

try:
    return abtool.getThumbnail(context, thumbnail_id, request)
except:
    raise NotFound, "Unknown page."


# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: pc_article.py 5777 2007-01-21 17:20:27Z roeder $

from funittest import physical
from funittest import logical
from funittest import interpreter
from funittest import dataprovider
from funittest import register_spt
from funittest import Testable
import random
import os

class Article:

    def existing_article(self, article):        
        interpreter.annotate("Setup article")
        logical.cmfplone.navigation.top()
        logical.cmfplone.folder.add(article)
        logical.cmfplone.content.edit(article)        
        logical.cmfplone.content.save(article)

    def existing_article_without_attachments(self, article):        
        interpreter.annotate("Setup article")
        self.existing_article(article)
        # Need to delete all images
        for attachment_type in ["images", "files", "links"]:
            physical.cmfplone.tab.access("edit")
            physical.cmfplone.schemata.access(attachment_type)
            i = 0
            while interpreter.is_element_present("//div[@id='%s-innercontent%06d']" % (attachment_type, i)):
                interpreter.click("//div[@id='%s-innercontent%06d']/div/a[@class='file_delete']" % (attachment_type, i))
                i+=1
                #os.system("pause")
            logical.cmfplone.content.save(article)
        #os.system("pause")
        
    def existing_article_with_attachments(self, article, files, images, links):
        self.existing_article_without_attachments(article)
        if files:
            for file in files:
                logical.plonearticle.article.add_file(file)
        if images:
            for image in images:
                logical.plonearticle.article.add_image(image)
        if links:
            for link in links:
                logical.plonearticle.article.add_link(link)        
        logical.plonearticle.article.save_article(article)    
        
register_spt("PloneArticle", Article())




# -*- coding: utf-8 -*-
## Test case for all InnerContentField classes
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for all InnerContentField classes
"""

__docformat__ = 'restructuredtext'

# Python imports
from copy import deepcopy
from types import ListType

# CMF imports
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.tests.utils import mkDummyInContext
from Products.Archetypes.tests.test_fields import FakeRequest
from Products.Archetypes.public import Schema, BaseFolderMixin

# Products imports
from Products.PloneArticle.tests.common import BaseTestCase, loadFile
from Products.PloneArticle import field as pa_fields
from Products.PloneArticle import proxy as pa_proxies

# we do as archetypes do for testing fields
test_fields = [
    ('FileInnerContentField', 'fileinnercontentfield'),
    ('ImageInnerContentField', 'imageinnercontentfield'),
    ('LinkInnerContentField', 'linkinnercontentfield'),
    ]

field_instances = []
for type_name, name in test_fields:
    field_instances.append(getattr(pa_fields, type_name)(name))

field_values = {
    'fileinnercontentfield': (
        {'id': 'object_1',
         'title': 'Title 1',
         },
        {'id': 'object_2',
         'title': 'Title 2',
         },
    ),
    'linkinnercontentfield': (
        {'id': 'object_1',
         'title': 'Title 1',
         },
        {'id': 'object_2',
         'title': 'Title 2',
         },
    ),
    'imageinnercontentfield': (
        {'id': 'object_1',
         'title': 'Title 1',
         },
        {'id': 'object_2',
         'title': 'Title 2',
         },
    ),
}

expected_values = {
    'fileinnercontentfield': (
        {'id': 'object_1',
         'title': 'Title 1',
         },
        {'id': 'object_2',
         'title': 'Title 2',
         },
    ),
    'imageinnercontentfield': (
        {'id': 'object_1',
         'title': 'Title 1',
         },
        {'id': 'object_2',
         'title': 'Title 2',
         },
    ),
    'linkinnercontentfield': (
        {'id': 'object_1',
         'title': 'Title 1',
         },
        {'id': 'object_2',
         'title': 'Title 2',
         },
    ),
}

empty_values = {
    'fileinnercontentfield': (),
    'imageinnercontentfield': (),
    'linkinnercontentfield': (),
    }

schema = Schema(tuple(field_instances))

proxy_classes = {
    'fileinnercontentfield': pa_proxies.FileInnerContentProxy,
    'imageinnercontentfield': pa_proxies.ImageInnerContentProxy,
    'linkinnercontentfield': pa_proxies.LinkInnerContentProxy,
    }

proxy_schemas = {
    'fileinnercontentfield': pa_proxies.FileInnerContentProxySchema,
    'imageinnercontentfield': pa_proxies.ImageInnerContentProxySchema,
    'linkinnercontentfield': pa_proxies.LinkInnerContentProxySchema,
    }

proxy_references =  (
    ('fileinnercontentfield', 'File', 'file', 'ref file', ),
    ('imageinnercontentfield', 'Image', 'image', loadFile('test.jpg'), ),
    ('linkinnercontentfield', 'Link', 'remoteUrl', 'http://www.google.fr', ),)

proxy_attached_field_names = (
    ('fileinnercontentfield', 'attachedFile'),
    ('imageinnercontentfield', 'attachedImage'),
    ('linkinnercontentfield', None),)

class Dummy(BaseFolderMixin):
    def Title(self):
        return 'Spam'

class InnerContentFieldTestCase(BaseTestCase):
    """
    This class contains all test cases for all InnerContentField classes
    """

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self._dummy = mkDummyInContext(Dummy, oid='dummy', context=self.portal,
                                       schema=schema)

    def makeDummy(self):
        return self._dummy

    def test_processing(self):
        dummy = self.makeDummy()
        request = FakeRequest()
        request.form.update(field_values)
        dummy.REQUEST = request
        dummy.processForm(data=1)
        for k, v in expected_values.items():
            field = dummy.getField(k)
            got = dummy.getField(k).get(dummy)

            if isinstance(field, pa_fields.BaseInnerContentField):
                self.assertEquals(
                    type(got), ListType,
                    'Bad type for BaseInnerContentField: expected %s, got %s' % (ListType, type(got)))

                result_list = []
                for val in got:
                    self.failUnless(
                        isinstance(val, pa_proxies.BaseInnerContentProxy))
                    val = {'id': val.getId(), 'title': val.Title(),}
                    result_list.append(val)
                got = tuple(result_list)

            self.assertEquals(got, v, 'got: %r, expected: %r, field "%s"' %
                              (got, v, k))

    def test_inner_content_reorder(self):
        dummy = self.makeDummy()
        field_name = 'fileinnercontentfield'
        file_values = ({'id': 'reorder_object_1',
                        'title': 'Title 1',},
                       {'id': 'reorder_object_2',
                        'title': 'Title 2',
                        },)

        expected_values = ({'id': 'reorder_object_1',
                            'title': 'Title 1',},
                           {'id': 'reorder_object_2',
                            'title': 'Title 2',
                            },)

        field = dummy.getField(field_name)
        field.set(dummy, file_values)
        got = dummy.getField(field_name).get(dummy)

        result_list = []
        for val in got:
            val = {'id': val.getId(), 'title': val.Title(),}
            result_list.append(val)
        got = tuple(result_list)

        self.assertEquals(got, expected_values,
                          'error on set, cannot test reordering'
                          'got: %r, expected: %r, field "%s"' %
                          (got, expected_values, field_name))

        file_values = list(file_values)
        file_values.reverse()
        expected_values = list(expected_values)
        expected_values.reverse()
        field.set(dummy, file_values)
        got = dummy.getField(field_name).get(dummy)

        result_list = []
        for val in got:
            val = {'id': val.getId(), 'title': val.Title(),}
            result_list.append(val)
        got = result_list

        self.assertEquals(got, expected_values,
                          'field inner content reorder failed, '
                          'got: %r, expected: %r, field "%s"' %
                          (got, expected_values, field_name))

    def test_inner_content_update(self):
        dummy = self.makeDummy()
        field_name = 'fileinnercontentfield'
        file_values = ({'id': 'reorder_object_1',
                        'title': 'Title 1',},
                       {'id': 'reorder_object_2',
                        'title': 'Title 2',
                        },)

        expected_values = ({'id': 'reorder_object_1',
                            'title': 'Title 1',},
                           {'id': 'reorder_object_2',
                            'title': 'Title 2',
                            },)

        field = dummy.getField(field_name)
        field.set(dummy, file_values)
        got = dummy.getField(field_name).get(dummy)

        result_list = []
        for val in got:
            val = {'id': val.getId(), 'title': val.Title(),}
            result_list.append(val)
        got = tuple(result_list)

        self.assertEquals(got, expected_values,
                          'error on set, cannot test reordering'
                          'got: %r, expected: %r, field "%s"' %
                          (got, expected_values, field_name))

        update_values = ({'id': 'reorder_object_3',
                        'title': 'Title 3',},
                       {'id': 'reorder_object_4',
                        'title': 'Title 4',
                        },)

        expected_values = (
                           {'id': 'reorder_object_1',
                            'title': 'Title 1',},
                           {'id': 'reorder_object_2',
                            'title': 'Title 2',},
                           {'id': 'reorder_object_3',
                           'title': 'Title 3',},
                           {'id': 'reorder_object_4',
                            'title': 'Title 4',},)


        field.set(dummy, update_values, update=True)
        got = dummy.getField(field_name).get(dummy)

        result_list = []
        for val in got:
            val = {'id': val.getId(), 'title': val.Title(),}
            result_list.append(val)
        got = tuple(result_list)

        self.assertEquals(got, expected_values,
                          'error on set, cannot test reordering'
                          'got: %r, expected: %r, field "%s"' %
                          (got, expected_values, field_name))

        update_values = ({'id': 'reorder_object_3',
                        'title': 'Title 3.1',},)

        expected_values = (
                           {'id': 'reorder_object_1',
                            'title': 'Title 1',},
                           {'id': 'reorder_object_2',
                            'title': 'Title 2',},
                           {'id': 'reorder_object_3',
                           'title': 'Title 3.1',},
                           {'id': 'reorder_object_4',
                            'title': 'Title 4',},)


        field.set(dummy, update_values, update=True)
        got = dummy.getField(field_name).get(dummy)

        result_list = []
        for val in got:
            val = {'id': val.getId(), 'title': val.Title(),}
            result_list.append(val)
        got = tuple(result_list)

        self.assertEquals(got, expected_values,
                          'error on set, cannot test reordering'
                          'got: %r, expected: %r, field "%s"' %
                          (got, expected_values, field_name))

    def testPortalCatalog(self):
        dummy = self.makeDummy()
        ctool = getToolByName(self.portal, 'portal_catalog')
        for k, v in proxy_classes.items():
            field = dummy.getField(k)
            proxy_type = v.portal_type

            # Add proxies
            value = field_values[k]
            field.set(dummy, value)
            brains = ctool(portal_type=proxy_type)
            self.assertEquals(len(brains), 2)

            # Add the same proxies to make sure proxies are not duplicated
            field.set(dummy, value)
            brains = ctool(portal_type=proxy_type)
            self.assertEquals(len(brains), 2)

            # Delete proxies
            field.set(dummy, ())
            brains = ctool(portal_type=proxy_type)
            self.assertEquals(len(brains), 0)

            # Add temporary object
            brains = ctool(portal_type=proxy_type)
            brain_length = len(brains)
            tmp_obj = field.getTemporaryInnerContent(dummy)
            brains = ctool(portal_type=proxy_type)
            self.assertEquals(len(brains), brain_length)

    def testUIDCatalog(self):
        dummy = self.makeDummy()
        utool = getToolByName(self.portal, 'uid_catalog')
        for k, v in proxy_classes.items():
            field = dummy.getField(k)
            proxy_type = v.portal_type

            # Add proxies
            value = field_values[k]
            field.set(dummy, value)
            brains = utool(portal_type=proxy_type)
            self.assertEquals(len(brains), len(value))

            # Add the same proxies to make sure proxies are not duplicated
            field.set(dummy, value)
            brains = utool(portal_type=proxy_type)
            self.assertEquals(len(brains), len(value))

            # Delete proxies
            field.set(dummy, ())
            brains = utool(portal_type=proxy_type)
            self.assertEquals(len(brains), 0)

            # Add temporary object
            brains = utool(portal_type=proxy_type)
            brain_length = len(brains)
            tmp_obj = field.getTemporaryInnerContent(dummy)
            brains = utool(portal_type=proxy_type)
            self.assertEquals(len(brains), brain_length)

    def testReferenceCatalog(self):
        dummy = self.makeDummy()
        utool = getToolByName(self.portal, 'uid_catalog')
        rtool = getToolByName(self.portal, 'reference_catalog')
        for k, ref_type, ref_field, ref_value in proxy_references:
            field = dummy.getField(k)

            # Create referenced content
            kwargs = {}
            kwargs[ref_field] = ref_value
            ref_content = self.addContent(ref_type, self.portal, k)
            ref_content.edit(**kwargs)
            ref_content_uid = ref_content.UID()

            # Add proxies with a reference
            value = deepcopy(field_values[k])

            for item in value:
                item['referencedContent'] = ref_content_uid
            field.set(dummy, value)
            brains = utool(portal_type='Reference')
            self.assertEquals(len(brains), len(value))
            brains = rtool(targetUID=ref_content_uid)
            self.assertEquals(len(brains), len(value))


            # Add the same proxies to make sure proxies are not duplicated
            field.set(dummy, value)
            brains = utool(portal_type='Reference')
            self.assertEquals(len(brains), len(value))
            brains = rtool(targetUID=ref_content_uid)
            self.assertEquals(len(brains), len(value))

            # Delete proxies
            field.set(dummy, ())
            brains = utool(portal_type='Reference')
            self.assertEquals(len(brains), 0)

    def testGetTemporaryInnerContent(self):
        """Test field.BaseInnerContentField.getTemporaryInnerContent method"""
        dummy = self.makeDummy()
        for k, v in proxy_classes.items():
            field = dummy.getField(k)
            tmp_obj = field.getTemporaryInnerContent(dummy)
            self.failUnless(isinstance(tmp_obj, v))

    def testGetInnerContentSchema(self):
        """Test field.BaseInnerContentField.getInnerContentSchema method"""
        dummy = self.makeDummy()
        for k, v in proxy_schemas.items():
            field = dummy.getField(k)
            got_schema = field.getInnerContentSchema(dummy)
            self.assertEquals(got_schema, v)

    def testGetInnerContentAttachedField(self):
        """Test field.BaseInnerContentField.getInnerContentAttachedField"""
        dummy = self.makeDummy()
        for field_name, attached_name in proxy_attached_field_names:
            field = dummy.getField(field_name)
            result = field.getInnerContentAttachedField(dummy)
            if result is not None:
                result = result.getName()
            self.assertEquals(result, attached_name)

    def test_getIndexableValue(self):
        """Test if field returns the correct value for SearchableText index"""

        dummy = self.makeDummy()
        field_name = 'fileinnercontentfield'
        file_values = ({'id': 'object_1',
                        'title': 'Title 1',
                        'attachedFile': 'This is a dummy text'},
                       {'id': 'object_2',
                        'title': 'Title 2',
                        },)
        expected_value = "1 2 a dummy is text this title title"
        field = dummy.getField(field_name)
        field.set(dummy, file_values)
        value = field.getIndexableValue(dummy)
        value_list = [x.lower() for x in value.split(" ") if x]
        value_list.sort()
        value = " ".join(value_list)
        self.assertEquals(value, expected_value)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(InnerContentFieldTestCase))
    return suite



# -*- coding: utf-8 -*-
##
## Copyright (C) 2005-2007 Ingeniweb
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Misc import steps for GenericSetup (see profiles/default/import-steps.xml)
$Id: ExFile.py 6229 2007-08-02 13:30:44Z glenfant $
"""
__docformat__ = 'restructuredtext'
__author__ = ''

from zope.interface import implements
from Products.CMFQuickInstallerTool.interfaces import INonInstallable
from Products.CMFEditions.StandardModifiers import manage_addOMInsideChildrensModifier
from Products.ATContentTypes.configuration import zconf as atct_zconf
from Products.PloneArticle.config import PROJECTNAME, PLONEARTICLE_TOOL
from Products.CMFCore.utils import getToolByName

class HiddenProductsNProfiles(object):
    implements(INonInstallable)

    def getNonInstallableProfiles(self):
        return [] #[u'Products.PloneArticle:default',]

    def getNonInstallableProducts(self):
        # hides "Products.PloneArticle": only "PloneArticle"
        return ['Products.PloneArticle',]


def thisProfileOnly(func):
    """Decorator that prevents the setup func to be used on other GS profiles.
    Usage:
    @thisProfileOnly
    def someFunc(context): ...
    """

    def wrapper(context):
        if context.readDataFile('plonearticle.txt') is None:
            return
        else:
            return func(context)
    return wrapper


@thisProfileOnly
def setupTool(context):
    """ Create PloneArticleTool at site root """

    site = context.getSite()

    atool = getattr(site, PLONEARTICLE_TOOL, None)

    if atool is None:
        site.manage_addProduct[PROJECTNAME].manage_addTool('PloneArticleTool')
        atool = getattr(site, PLONEARTICLE_TOOL, None)
        atool.setVersionFromFS()

    numvers, version = atool.getVersion()
    if not numvers and not version:
        # we are reinstalling over an older version. Set to the first one from
        # which we can migrate
        atool.setInstanceVersion('3.2.99')

    return 'PloneArticleTool  created'


@thisProfileOnly
def setupModifier(context):
    """Add a modifier for PloneArticle for CMFEditions"""

    mod_id = 'PloneArticleModifier'
    portal_modifier = context.getSite().portal_modifier
    if mod_id in portal_modifier.objectIds():
        # We delete and recreate that modifier
        portal_modifier._delObject(mod_id)
    manage_addOMInsideChildrensModifier(portal_modifier, mod_id)
    modifier = getattr(portal_modifier, mod_id)
    modifier.edit(enabled=True, condition="python:meta_type == 'PloneArticle'")
    return "CMFEdition modifier for PloneArticle added"


@thisProfileOnly
def setupKupu(context):
    """Update kupu properties"""

    # Should be done by GS with kupu.xml but exportimport.py of kupu supports only full configuration

    site = context.getSite()
    ktool = getattr(site, 'kupu_library_tool', None)
    typestool = getToolByName(site, 'portal_types')
    if ktool and typestool:

        available_types = typestool.listContentTypes()
        kupu_resources = {
            'linkable': ('PloneArticle','ImageInnerContentProxy','FileInnerContentProxy'),
            'mediaobject': ('ImageInnerContentProxy',),
            'containsanchors': ('PloneArticle',),
            'collection': ('PloneArticle', 'InnerContentContainer')
            }

        for resource_type in ('linkable', 'mediaobject', 'containsanchors', 'collection'):
            new_types = list(ktool.getPortalTypesForResourceType(resource_type))
            
            # avoid bugs with ktool.getPortalTypesForResourceType
            # sometimes unavailable types can be always registered after a product uninstall
            for type_id in new_types :
                if type_id not in available_types :
                    new_types.remove(type_id)
                    print 'contenttype removed from kupu resource %s : %s \n' %(resource_type, type_id )
                    
            for type_id in kupu_resources[resource_type]:
                if type_id in new_types:
                    continue 
                new_types.append(type_id)
            new_resources = []
            new_resources.append({
                'old_type': resource_type,
                'resource_type': resource_type,
                'portal_types': new_types})
            ktool.updateResourceTypes(new_resources)
        return 'Kupu set for PloneArticle'
    else:
        return 'Kupu not installed'

DEFAULT_MAX_SIZE = 3 * 2 ** 20

@thisProfileOnly
def setupFTI(context):
    """Sets FTI max size default props as in ATCT config"""

    site = context.getSite()
    try:
        fti = site.portal_types.PloneArticle
    except AttributeError, e:
        return "PloneArticle FTI not installed in portal_types"

    fti.attachmentMaxSize = int(atct_zconf.ATFile.max_file_size) or DEFAULT_MAX_SIZE
    fti.imageMaxSize = int(atct_zconf.ATImage.max_file_size) or DEFAULT_MAX_SIZE
    return "PlonArticle FTI has ATCT defaults"



# -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import transaction
import itertools

# Zope imports
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.utils import getToolByName

from Products.PloneArticle import tool, content


# Module aliases needed for migration
class Dummy:
    pass


class OrderedList(SimpleItem):
    """Ordered list with unique elements"""

    security = ClassSecurityInfo()

    security.declarePrivate('__init__')
    def __init__(self):
        """Initialize list"""
        self.olist = []

    security.declarePrivate('add')
    def add(self, items):
        """Add new items"""
        for item in items:
            self.olist.append(item)

        self.olist = self.olist

    security.declarePrivate('remove')
    def remove(self, items):
        """Remove items"""
        for item in items:
            self.olist.remove(item)

        self.olist = self.olist

    security.declarePrivate('moveByDelta')
    def moveByDelta(self, items, delta):
        """Move items by delta"""

        for item in items:
            index = self.olist.index(item) - delta
            self.olist.remove(item)
            self.olist.insert(index, item)

        self.olist = self.olist

    security.declarePrivate('getItems')
    def getItems(self):
        """Get all items"""
        return self.olist

InitializeClass(OrderedList)

import new
attachmentmixin = new.module('AttachmentMixin')
attachmentmixin.AttachmentReferenceBrain = Dummy
attachmentmixin.OrderedList = OrderedList
attachmentmixin.AttachmentContents = Dummy

articlecore = new.module('ArticleCore')
articlecore.ArticleCore = Dummy
articlecore.ArticleReferenceBrain = Dummy

sys.modules['Products.PloneArticle.PloneArticle'] = content.article
sys.modules['Products.PloneArticle.PloneArticleMultiPage'] = content.multipage
sys.modules['Products.PloneArticle.PloneArticleTool'] = tool
sys.modules['Products.PloneArticle.AttachmentMixin'] = attachmentmixin
sys.modules['Products.PloneArticle.ArticleCore'] = articlecore

def applyArticles(portal, func, portal_type='PloneArticle',
                  savepoint_interval=100):
    """
    Apply a function on all articles

    @param portal_type: a string or list of strings for types to look for

    @param savepoint_interval: do a savepoint(optimistic) every 'interval'
    processed content
    """

    savepoint = transaction.savepoint
    portal_catalog = getToolByName(portal, 'portal_catalog')
    article_brains = portal_catalog(portal_type=portal_type)

    for brain, count in itertools.izip(article_brains, itertools.count()):
        article = brain.getObject()
        #FIXME: is article = None possible?
        func(article)

        if count % savepoint_interval:
            savepoint(optimistic=True)

    return

#####################################


def null(portal):
    """ This is a null migration, use it when nothing happens """
    pass

def executeMigrations():
    from Products.PloneArticle.migration import v4
    from Products.PloneArticle.migration import v4_1

def registerMigrations():

    tool.registerUpgradePath('3.2.99', '4.0.0-beta1', v4.betas.v3_v4beta1)
    tool.registerUpgradePath('4.0.0-beta1', '4.0.0-beta2', null)
    tool.registerUpgradePath('4.0.0-beta2', '4.0.0-beta3', null)
    tool.registerUpgradePath('4.0.0-beta3', '4.0.0-beta4', v4.betas.beta3_beta4)
    tool.registerUpgradePath('4.0.0-beta4', '4.0.0-beta5', null)
    tool.registerUpgradePath('4.0.0-beta5', '4.0.0-beta6', v4.betas.beta5_beta6)
    tool.registerUpgradePath('4.0.0-beta6', '4.0.0-rc1', null)
    tool.registerUpgradePath('4.0.0-rc1', '4.0.0-rc2', null)
    tool.registerUpgradePath('4.0.0-rc2', '4.0.0-rc3', null)
    tool.registerUpgradePath('4.0.0-rc3', '4.0.0-rc4', null)
    tool.registerUpgradePath('4.0.0-rc4', '4.0.0-rc5', null)
    tool.registerUpgradePath('4.0.0-rc5', '4.0.0-rc6', null)
    tool.registerUpgradePath('4.0.0-rc6', '4.0.0', null)
    tool.registerUpgradePath('4.0.0', '4.0.1', null)
    tool.registerUpgradePath('4.0.1', '4.0.2', null)
    tool.registerUpgradePath('4.0.2', '4.0.3', null)
    tool.registerUpgradePath('4.0.3', '4.0.4', null)
    tool.registerUpgradePath('4.0.4', '4.1.0-beta1', v4_1.betas.stable40x_410beta1)
    tool.registerUpgradePath('4.1.0-beta1', '4.1.0-beta2', null)
    tool.registerUpgradePath('4.1.0-beta2', '4.1.0-beta3', null)
    tool.registerUpgradePath('4.1.0-beta3', '4.1.0-RC1', null)
    tool.registerUpgradePath('4.1.0-RC1', '4.1.0-RC2', null)
    tool.registerUpgradePath('4.1.0-RC2', '4.1.0-final', null)
    tool.registerUpgradePath('4.1.0-final', '4.1.1', null)
    tool.registerUpgradePath('4.1.1', '4.1.2', null)
    tool.registerUpgradePath('4.1.2', '4.1.3', null)




# -*- coding: utf-8 -*-
## Used to install PloneArticle content
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module is used by portal_quickinstaller to install product
"""

__docformat__ = 'restructuredtext'

# Python imports
from StringIO import StringIO

# CMF
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions as CCP

# Archetypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes.public import listTypes

# Products import
from Products.PloneArticle.config import PROJECTNAME, GLOBALS, PLONEARTICLE_TOOL


# Configlets
plonearticle_management_configlet = {
    'id': 'plonearticle_management',
    'appId': PROJECTNAME,
    'name': 'PloneArticle preferences',
    'action': 'string:$portal_url/plonearticle_tool',
    'category': 'Products',
    'permission': (CCP.ManagePortal,),
    'imageUrl': 'plonearticle_tool.gif',
    }
    
types_to_hide = (
    'FileInnerContentProxy',
    'ImageInnerContentProxy',
    'LinkInnerContentProxy',
    'InnerContentContainer',
    'PloneArticleTool',)
    
kupu_resources = {
        'linkable': ('FileInnerContentProxy',),
        'mediaobject': ('ImageInnerContentProxy',),
        'collection': ('PloneArticle', 'InnerContentContainer',)
    }

# Installation is now done via GenericSetup

# def install(self):
#     """Install PloneArticle product"""
    
#     out = StringIO()

#     # Install types
#     typeInfo = listTypes(PROJECTNAME)
#     installTypes(self, out,
#                  typeInfo,
#                  PROJECTNAME)

#     # Install skin
#     install_subskin(self, out, GLOBALS)

#     # inner content setup

#     # no wf chain
#     pw = getToolByName(self, 'portal_workflow')
#     pw.setChainForPortalTypes(types_to_hide, '')
    
#     # Update portal properties
#     ptool = getToolByName(self, 'portal_properties')
    
#     # Update site_properties.types_not_searched property
#     sprops = ptool.site_properties
#     new_types = list(sprops.types_not_searched)
#     for type_id in types_to_hide:
#         if type_id in new_types:
#             continue
#         new_types.append(type_id)
#     sprops.types_not_searched = new_types
    
#     # Update site_properties.metaTypesNotToList property
#     nprops = ptool.navtree_properties
#     new_types = list(nprops.metaTypesNotToList)
#     for type_id in types_to_hide:
#         if type_id in new_types:
#             continue
#         new_types.append(type_id)
#     nprops.metaTypesNotToList = new_types

#     # don't catalog the tool
#     atool = getToolByName(self, 'archetype_tool')
#     atool.setCatalogsByType('PloneArticleTool', ())
    
#     # Update kupu properties
#     ktool = getToolByName(self, 'kupu_library_tool')
#     for resource_type in ('linkable', 'mediaobject', 'collection'):
#         new_types = list(ktool.getPortalTypesForResourceType(resource_type))
#         for type_id in kupu_resources[resource_type]:
#             if type_id in new_types:
#                 continue
#             new_types.append(type_id)
#         new_resources = []
#         new_resources.append({
#             'old_type': resource_type, 
#             'resource_type': resource_type, 
#             'portal_types': new_types})    
#         ktool.updateResourceTypes(new_resources)
    
#     # Install tool
#     atool = getattr(self, PLONEARTICLE_TOOL, None)
#     if atool is None:
#         self.manage_addProduct[PROJECTNAME].manage_addTool('PloneArticleTool')
#         atool = getattr(self, PLONEARTICLE_TOOL, None)
#         atool.setVersionFromFS()

#     numvers, version = atool.getVersion()
#     if not numvers and not version:
#         # we are reinstalling over an older version. Set to the first one from
#         # which we can migrate
#         atool.setInstanceVersion('3.2.99')
        
#     # Install configlet
#     cptool = getToolByName(self, 'portal_controlpanel')
#     try:
#         cptool.registerConfiglet(**plonearticle_management_configlet)
#     except:
#         pass
        
#     # Add portal types to portal factory
#     ftool = getToolByName(self, 'portal_factory')
#     types_to_add = (
#         'PloneArticle', 
#         'FileInnerContentProxy', 
#         'ImageInnerContentProxy',
#         'LinkInnerContentProxy',)
#     ftypes = ftool.getFactoryTypes()
#     ftypes.update(dict([(x, 1) for x in types_to_add]))
#     ftool.manage_setPortalFactoryTypes(listOfTypeIds=ftypes.keys())
#     out.write("Types configured to use portal_factory\n")
    
#     out.write('Installation completed.\n')
#     return out.getvalue()

def uninstall(self):
    """Uninstall PloneArticle product"""
    
    out = StringIO()
    
    # Update portal properties
    ptool = getToolByName(self, 'portal_properties')
    
    # Update site_properties.types_not_searched property
    sprops = ptool.site_properties
    new_types = [x for x in sprops.types_not_searched if x not in types_to_hide]
    sprops.types_not_searched = new_types
    
    # Update site_properties.metaTypesNotToList property
    nprops = ptool.navtree_properties
    new_types = [x for x in nprops.metaTypesNotToList if x not in types_to_hide]
    nprops.metaTypesNotToList = new_types
    
    # Update kupu properties
    ktool = getToolByName(self, 'kupu_library_tool')
    typestool = getToolByName(self, 'portal_types')
    
    availables_types = typestool.listContentTypes()
    
    for resource_type in ('linkable', 'mediaobject', 'collection'):
        new_types = [x for x in ktool.getPortalTypesForResourceType(resource_type) if x in availables_types and  x not in kupu_resources[resource_type]]
        new_resources = []
        new_resources.append({
            'old_type': resource_type, 
            'resource_type': resource_type, 
            'portal_types': new_types})    
        ktool.updateResourceTypes(new_resources)
    
    # Uninstall configlets
    try:
        cptool = getToolByName(self, 'portal_controlpanel')
        cptool.unregisterApplication(PROJECTNAME)
    except:
        pass
    
    out.write('Uninstallation completed.\n')
    return out.getvalue()



## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id$

# KSS Server action to edit proxies : No more used

from zope.interface import implements

from kss.core import KSSView, kssaction

from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile

class PAKssEdit(KSSView):
    
    model_wrapper = ZopeTwoPageTemplateFile('templates/model_wrapper.pt')
    
    @kssaction
    def replaceModelContent (self, modelname):
        """  Used to replace all content after inline editing  """        
           
        content = self.model_wrapper(pamacro = 'here/%s/macros/main' %modelname ,
                                     context = self.context)
        core = self.getCommandSet('core')
        core.replaceInnerHTML('#pacontent', content)



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: test_add_link.py 5782 2007-01-21 17:57:11Z roeder $

__author__  = 'Maik Roeder <maik.roeder@ingeniweb.com>'
__docformat__ = 'restructuredtext'

import unittest
from funittest import logical
from funittest import dataprovider
from funittest import scripts
from funittest import interpreter
from funittest.testrunner import Test
from funittest import Schema
from funittest import register_tst

import os

class AddLink(Test):
    schema = Schema()

    def setUp(self):    
        interpreter.open("")        
        user=dataprovider.cmfplone.user.get('sampleadmin')
        logical.cmfplone.application.login(user)        
        self.article = dataprovider.plonearticle.article.get("article1")
        scripts.plonearticle.article.existing_article(self.article)

    def step_1(self):
        "Add a link to an article and save the article"
        interpreter.annotate("Test: Add a link to the article and save the article")
        link = dataprovider.plonearticle.articlelink.get("Link 1")
        logical.plonearticle.article.add_link(link)
        logical.plonearticle.article.save_article(self.article)
        
    def test(self):
        self.expect_ok(1)

register_tst("PloneArticle", AddLink())



# -*- coding: utf-8 -*-
## This module contains permissons used in PloneArticle product
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# Zope imports
from Globals import InitializeClass
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation
from Products.CMFCore.TypesTool import TypesTool, FactoryTypeInformation

# Products imports
from Products.PloneArticle import utils
from Products.PloneArticle.interfaces import IBaseInnerContent
from Products.PloneArticle.proxy.fileinnercontent import FileInnerContentProxy
from Products.PloneArticle.proxy.imageinnercontent import ImageInnerContentProxy
from Products.PloneArticle.proxy.linkinnercontent import LinkInnerContentProxy

fti_meta_type = 'Factory-based Type Information for Plone Articles content types'

class PloneArticleFactoryTypeInformation(DynamicViewTypeInformation):
    """FTI for PloneArticle based types options
        PloneArticle FactoryTypeInformation is a dynamic view FTI with some other
        properties enabling it to store PA options per content type in a clean way.

        Properties are :
        * referenceableAttachmentType
        * referenceableImageType
        * referenceableLinkType
        * attachmentMaxSize
        * imageMaxSize
        * previewAllowed
    """

    meta_type = fti_meta_type
    security = ClassSecurityInfo()

    referenceableAttachmentType = []
    _properties = DynamicViewTypeInformation._properties + (
        {
            'id': 'referenceableAttachmentType',
            'type': 'multiple selection',
            'mode': 'w',
            'label': 'Referenceable attachment type',
            'select_variable': 'getAvailableReferenceableAttachmentTypes',
        },
        {
            'id': 'referenceableImageType',
            'type': 'multiple selection',
            'mode': 'w',
            'label': 'Referenceable image type',
            'select_variable': 'getAvailableReferenceableImageTypes',
        },
        {
            'id': 'referenceableLinkType',
            'type': 'multiple selection',
            'mode': 'w',
            'label': 'Referenceable link type',
            'select_variable': 'getAvailableReferenceableLinkTypes',
        },
        {
            'id': 'attachmentMaxSize',
            'type': 'int',
            'mode': 'w',
            'label': 'Max size for attachment',
        },
        {
            'id': 'imageMaxSize',
            'type': 'int',
            'mode': 'w',
            'label': 'Max size for image',
        },
        {
            'id': 'previewAllowed',
            'type': 'boolean',
            'mode': 'w',
            'label': 'Is preview allowed?',
        },

    )

    referenceableAttachmentType = []
    referenceableImageType = []
    referenceableLinkType = []
    attachmentMaxSize = 3 * 2 ** 20
    imageMaxSize = 3 * 2 ** 20
    previewAllowed = True

    def manage_afterAdd(self, item, container):
        ## This method will only work with third party products. Base PloneArticle
        ## initialize the FTI with another method.
        try:
            paTool = getToolByName(self, 'plonearticle_tool')
        except AttributeError:
            return

        self.initializeFti(item)

    def initializeFti(self, item):
        """
            Initialize the FTI with default value
            This method can be called only when the PA tool is installed.
        """

        self.referenceableAttachmentType = getattr(self, 'referenceableAttachmentType', self.getAvailableReferenceableAttachmentTypes())
        self.referenceableImageType = getattr(self, 'referenceableImageType', self.getAvailableReferenceableImageTypes())
        self.referenceableLinkType = getattr(self, 'referenceableLinkType', self.getAvailableReferenceableLinkTypes())
        self.attachmentMaxSize = getattr(self, 'attachmentMaxSize', 3 * 2 ** 20)
        self.imageMaxSize = getattr(self, 'imageMaxSize', 3 * 2 ** 20)
        self.previewAllowed = getattr(self, 'previewAllowed', True)

    def getAvailableReferenceableAttachmentTypes(self):
        return utils.getAllAvailableReferenceableTypes(self, FileInnerContentProxy)

    def getAvailableReferenceableImageTypes(self):
        return utils.getAllAvailableReferenceableTypes(self, ImageInnerContentProxy)

    def getAvailableReferenceableLinkTypes(self):
        return utils.getAllAvailableReferenceableTypes(self, LinkInnerContentProxy)

InitializeClass(PloneArticleFactoryTypeInformation)


def manage_addPAFTIForm(self, REQUEST):
    """ Get the add form for factory-based type infos.
    """
    addTIForm = DTMLFile('addTypeInfo', _dtmldir).__of__(self)
    ttool = getToolByName(self, 'portal_types')
    return addTIForm( self, REQUEST,
                      add_meta_type=PloneArticleFactoryTypeInformation.meta_type,
                      types=ttool.listDefaultTypeInformation() )

class DynamicAllowedContentFTI(FactoryTypeInformation):
    """
    Allow content to be added if it implements BaseInnercontentProxy
    """

    meta_type = "DynamicAllowedContentFTI for InnerContentContainer"

    security = ClassSecurityInfo()

    security.declarePublic('allowType')
    def allowType(self, contentType):
        """
            Can objects of 'contentType' be added to containers whose
            type object we are?
        """
        atool = getToolByName(self, 'archetype_tool')
        types = [ti for ti in atool.listRegisteredTypes()
                 if ti['portal_type'] == contentType]
        return len(types) > 0 and \
               atool.typeImplementsInterfaces(types[0], (IBaseInnerContent,))

InitializeClass(DynamicAllowedContentFTI)

def manage_addDVTFTIForm(self, REQUEST):
    """ Get the add form for factory-based type infos.
    """
    addTIForm = DTMLFile('addTypeInfo', _dtmldir).__of__(self)
    ttool = getToolByName(self, 'portal_types')
    return addTIForm( self, REQUEST,
                      add_meta_type=PloneArticleFactoryTypeInformation.meta_type,
                      types=ttool.listDefaultTypeInformation() )

# BBB: the following lines are required to register the new FTI in CMF 1.5 and may
# be removed after switching to CMF 1.6
try:
    from Products.CMFCore.TypesTool import typeClasses
except ImportError:
    pass
else:

    setattr(TypesTool, 'manage_addPAFTIForm',
                        manage_addPAFTIForm)

    setattr(TypesTool, 'manage_addPAFTIForm__roles__',
                        ('Manager', ))

    typeClasses.append(
        {'class' : PloneArticleFactoryTypeInformation,
         'name' : PloneArticleFactoryTypeInformation.meta_type,
         'action': 'manage_addPAFTIForm',
         'permission' : ManagePortal,
         },
        )

    setattr(TypesTool, 'manage_addDVTFTIForm',
                        manage_addDVTFTIForm)

    setattr(TypesTool, 'manage_addDVTFTIForm__roles__',
                        ('Manager', ))

    typeClasses.append(
        {'class' : DynamicAllowedContentFTI,
         'name' : DynamicAllowedContentFTI.meta_type,
         'action': 'manage_addDVTFTIForm',
         'permission' : ManagePortal,
         },
        )



# -*- coding: utf-8 -*-
## Defines BaseTestCase class. Inherits from this class to make test cases
## on PloneArticle product
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines BaseTestCase class. Inherits from this class to make test cases on
PloneArticle product
"""

__docformat__ = 'restructuredtext'

# Zope imports
from Testing import ZopeTestCase

# CMF imports
from Products.CMFCore.utils import getToolByName

# Plone imports
from Products.PloneTestCase import PloneTestCase
from Products.PloneTestCase.setup import PLONE21, PLONE25

class BaseTestCase(PloneTestCase.PloneTestCase):
    """Base class for all class with test cases on PloneArticle"""

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)

    def addContent(self, portal_type, container, content_id, **kwargs):
        """Add new content"""

        container.invokeFactory(portal_type, id=content_id, **kwargs)
        content = getattr(container, content_id)
        return content

    def addPloneArticle(self, container, content_id, **kwargs):
        """Add PloneArticle content"""

        return self.addContent('PloneArticle', container, content_id)

ZopeTestCase.installProduct('PlacelessTranslationService')

# additional content type
from Products.Archetypes.atapi import Schema, BaseContentMixin, registerType
from Products.PloneArticle.config import PROJECTNAME
from Products.PloneArticle.field import SmartListField

ALLOWED_CRITERIAS = ('SearchableText', 'portal_type',)
schema = Schema((
    SmartListField('smartlistfield',
                   relationship='unit_test',
                   auto_reference=True,
                   ),
    SmartListField('restricted_smartlistfield',
                   relationship='unit_test_restricted',
                   allowed_types=('PloneArticle',),
                   allowed_criterias=ALLOWED_CRITERIAS,
                   )
    ))

class SimpleSmartContent(BaseContentMixin):
    """Content with a smartlist field
    """
    schema = schema

registerType(SimpleSmartContent, PROJECTNAME)


# Install Attachment field if it exists
from Products.PloneArticle.tool import USE_ATTACHMENT_FIELD

if USE_ATTACHMENT_FIELD:
    ZopeTestCase.installProduct('AttachmentField')

# Install PloneArticle
ZopeTestCase.installProduct('PloneArticle')

# Setup Plone site
if USE_ATTACHMENT_FIELD:
    PloneTestCase.setupPloneSite(products=['kupu', 'AttachmentField', 'PloneArticle'])
else:
    PloneTestCase.setupPloneSite(products=['kupu', 'PloneArticle'])



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: dp_article.py 5766 2007-01-15 16:52:32Z roeder $
from funittest import DataProvider

class Article(DataProvider):
    pass
    
default = {'id':'article',
           'title':'An article',
           'description':'An article description',
           'text':'An article text',
           'type_name':'PloneArticle'}
key="id"
article = Article("PloneArticle", default, key)

provide = article.provide

provide({'id':'article1',
         'title':'Article 1',
         'description':"Article 1 Description",
         'text':'Article 1 Body Text',})



# -*- coding: utf-8 -*-
## Defines ImageInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines ImageInnerContentProxy
"""

__docformat__ = 'restructuredtext'

# Zope imports
from zope.interface import implements
from AccessControl import ClassSecurityInfo
from OFS.Image import Image

# CMF imports
from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.public import ImageField, ImageWidget, ReferenceField, \
    registerType, Schema, ComputedField, ComputedWidget

# Products imports
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import \
    ReferenceBrowserWidget

from Products.ATContentTypes.interface import IImageContent

from Products.PloneArticle import LOG
from Products.PloneArticle.proxy import BaseFileContentProxy, \
    BaseInnerContentProxySchema
from Products.PloneArticle.interfaces import IImageInnerContentProxy, IBaseInnerContentProxy
from Products.PloneArticle.config import PLONEARTICLE_TOOL


# Defines schema
ImageInnerContentProxySchema = BaseInnerContentProxySchema.copy() + Schema((
    ComputedField(
        'image',
        primary=True,
        expression="""context.getPrimaryValue('image', 'attachedImage', '')""",
        widget=ComputedWidget(
            label='Image',
            label_msgid='label_image',
            i18n_domain='plonearticle',
            ),
        ),
    ReferenceField(
        'referencedContent',
        relationship='article_image',
        keepReferencesOnCopy=True,
        widget=ReferenceBrowserWidget(
            label='Referenced image',
            label_msgid='label_referenced_image',
            i18n_domain='plonearticle',
            ),
        ),
    ImageField(
        'attachedImage',
        attached_content=True,
        widget=ImageWidget(
            label='Attached image',
            label_msgid='label_attached_image',
            i18n_domain='plonearticle',
            ),
        ),
    ))
    
ImageInnerContentProxySchema['title'].required = False

class ImageInnerContentProxy(BaseFileContentProxy):
    """Proxy implementing IImageContent. It means this proxy has a getImage 
    method.
    
    getImage returns attached image by default if existing otherwise returns
    the referenced content.
    """
    
    security = ClassSecurityInfo()
    implements(IImageInnerContentProxy)
    
    schema = ImageInnerContentProxySchema

    # You can only reference content implementing IImageContent interface
    referenceable_interfaces = (IImageContent,)
    
    security.declareProtected(CCP.View, 'index_html')
    def index_html(self, REQUEST=None, RESPONSE=None):
        """Make it directly viewable when entering the objects URL.
        
        We have to reproduce it to keep the same behaviour as usual for images.
        """
        
        if REQUEST is None:
            REQUEST = self.REQUEST
        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE
        
        field = self.getPrimaryField()
        accessor = field.getAccessor(self)
        data = accessor()
        
        if not isinstance(data, Image):
            return ''
        
        return data.index_html(REQUEST, RESPONSE)
    
    security.declareProtected(CCP.View, 'tag')
    def tag(self, **kwargs):
        """Use proxy title for img tags: title and alt"""
        
        # Scale image
        patool = getToolByName(self, PLONEARTICLE_TOOL)
        return patool.getThumbnailTag(self, 'image', **kwargs)
    

    def setAttachedImage(self, value, **kwargs):
        """
        Rename proxy according to file name
        """
        field = self.getField('attachedImage')
        field.set(self, value, **kwargs)
        self.renameFromFileName(field)

registerType(ImageInnerContentProxy)



# -*- coding: utf-8 -*-
## Defines FileInnerContentField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines FileInnerContentProxy
"""

__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from OFS.Image import File
from zope.interface import implements

# CMF imports
from Products.CMFCore import permissions as CCP

# Archetypes imports
from Products.Archetypes.public import StringField, \
    TextAreaWidget, registerType, Schema, ReferenceField, ComputedField, \
    ComputedWidget

# Products imports
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import \
    ReferenceBrowserWidget

from Products.ATContentTypes.interface import IFileContent
from Products.PloneArticle.proxy import BaseFileContentProxy, \
     BaseInnerContentProxySchema
from Products.PloneArticle.interfaces import IFileInnerContentProxy

# Use AttachmentField product if found otherwise use standard FileField
try:
    from Products.AttachmentField.AttachmentField import AttachmentField \
        as ProxyFileField
    from Products.AttachmentField.AttachmentWidget import AttachmentWidget \
        as ProxyFileWidget
except:
    from Products.Archetypes.public import FileField as ProxyFileField
    from Products.Archetypes.public import FileWidget as ProxyFileWidget
    
# Defines schema
FileInnerContentProxySchema = BaseInnerContentProxySchema.copy() + Schema((
    ComputedField(
        'file',
        primary=True,
        expression="""context.getPrimaryValue('file', 'attachedFile', '')""",
        widget=ComputedWidget(
            label='File',
            label_msgid='label_file',
            i18n_domain='plonearticle',
            ),
        ),
    ReferenceField(
        'referencedContent',
        relationship='article_file',
        keepReferencesOnCopy=True,
        widget=ReferenceBrowserWidget(
            label='Referenced file',
            label_msgid='label_referenced_file',
            i18n_domain='plonearticle',
            ),
        ),    
    ProxyFileField(
        'attachedFile',
        attached_content=True,
        searchable=True,
        widget=ProxyFileWidget(
            label='Attached file',
            label_msgid='label_attached_file',
            i18n_domain='plonearticle',
            ),
        ),
    ))

class FileInnerContentProxy(BaseFileContentProxy):
    """Proxy implementing IFileContent. It means this proxy has a getFile 
    method.
    
    getFile returns attached file by default if existing otherwise returns
    the referenced content.
    """
    
    implements(IFileInnerContentProxy)    
    security = ClassSecurityInfo()

    schema = FileInnerContentProxySchema

    # You can only reference content implementing IFileContent interface
    referenceable_interfaces = (IFileContent,)
    
    security.declareProtected(CCP.View, 'index_html')
    def index_html(self, REQUEST=None, RESPONSE=None):
        """Make it directly viewable when entering the objects URL.
        
        We have to reproduce it to keep the same behaviour as usual for files.
        """
        
        if REQUEST is None:
            REQUEST = self.REQUEST
        if RESPONSE is None:
            RESPONSE = REQUEST.RESPONSE
        field = self.getPrimaryField()
        accessor = field.getAccessor(self)
        data = accessor()
        
        if not isinstance(data, File):
            return ''
        
        if data.getContentType().startswith('text/'): 
            return data

        RESPONSE.setHeader(
            'Content-Disposition',
            'attachment; filename="%s"' % data.filename or self.getId())
        return data.index_html(REQUEST, RESPONSE)
    
    def setAttachedFile(self, value, **kwargs):
        """
        Rename proxy according to file name
        """
        field = self.getField('attachedFile')
        field.set(self, value, **kwargs)
        self.renameFromFileName(field)
    
registerType(FileInnerContentProxy)



## -*- coding: utf-8 -*-
## Copyright (C) 2008 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# $Id$

"""
Models params (styles ...)
"""

# images player config #
# MAXWIDTH used for preview images maxwidth & player width
MAXWIDTH = 300
# MAXHEIGHT used for preview images maxheight & player height
MAXHEIGHT = 200
# THUMBMAXWIDTH used for thumbs images maxwidth
THUMBMAXWIDTH = 40
# MAXHEIGHT used for thumbs images maxheight
THUMBMAXHEIGHT = 40
# CONTROLS ORIENTATION ('vertical' or 'horizontal')
CONTROLSORIENTATION = 'horizontal'
# for horizontal controls only ('top' or 'bottom')
HORIZONTALCONTROLSPOSITION = 'bottom'
# for vertical controls only ('left' or 'right')
VERTICALCONTROLSPOSITION = 'right'
# others styles
# SET DISABLESTYLES to True if you prefer to redefine these styles through css
DISABLESTYLES = False
FLOAT = 'right'
BORDERCOLOR = '#DDDCCC'
BGCOLOR = '#F6F6F6'
MARGIN = '0.5em 1em'



# -*- coding: utf-8 -*-
## Test case for SmartListField
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Test case for SmartListField
"""

__docformat__ = 'restructuredtext'

# Python imports
from copy import deepcopy
from types import ListType
from StringIO import StringIO

# CMF imports
from Products.CMFCore.utils import getToolByName

from Products.PloneTestCase.setup import PLONE21, PLONE25, PLONE30

# Archetypes imports
from Products.Archetypes.tests.utils import mkDummyInContext, makeContent
from Products.Archetypes.tests.test_fields import Dummy, FakeRequest
from Products.Archetypes.atapi import Schema
from Products.Archetypes.Registry import fieldDescriptionRegistry
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.atapi import listTypes

# Products imports
from Products.PloneArticle.tests.common import BaseTestCase, loadFile
from Products.PloneArticle.field import SmartListField
from Products.PloneArticle import proxy as pa_proxies
from Products.PloneArticle.config import PROJECTNAME

def lowcase_cmp(s1, s2):
    return cmp(s1.lower(), s2.lower())

if PLONE30:
    ALL_ALLOWED_CRITERIAS= (
        'allowedRolesAndUsers', 'Creator', 'getEventType', 'getId',
        'getObjPositionInParent', 'getRawRelatedItems', 'id', 'in_reply_to',
        'is_default_page', 'is_folderish', 'meta_type', 'object_provides',
        'path', 'portal_type', 'review_state', 'SearchableText',
        'sortable_title', 'Subject', 'Type', 'UID'
        )
elif PLONE25:
    ALL_ALLOWED_CRITERIAS = (
        'allowedRolesAndUsers', 'Creator', 'getEventType', 'getId',
        'getObjPositionInParent', 'getRawRelatedItems', 'id', 'in_reply_to',
        'is_default_page', 'is_folderish', 'meta_type', 'path', 'portal_type',
        'review_state', 'SearchableText', 'sortable_title', 'Subject', 'Type'
        )
else: # Plone 2.1 ?
    ALL_ALLOWED_CRITERIAS = (
        'allowedRolesAndUsers', 'Creator', 'getId',
        'getObjPositionInParent', 'getRawRelatedItems', 'id', 'in_reply_to',
        'is_default_page', 'is_folderish', 'meta_type', 'path', 'portal_type',
        'review_state', 'SearchableText', 'sortable_title', 'Subject', 'Type')

ALLOWED_CRITERIAS = ('SearchableText', 'portal_type',)

schema = Schema((
    SmartListField('smartlistfield',
                   relationship='unit_test',
                   auto_reference=True,
                   ),
    SmartListField('restricted_smartlistfield',
                   relationship='unit_test_restricted',
                   allowed_types=('PloneArticle',),
                   allowed_criterias=ALLOWED_CRITERIAS,
                   )
    ))


class SmartListFieldTestCase(BaseTestCase):
    """
    """

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self._dummy = mkDummyInContext(Dummy, oid='dummy',
                                       context=self.portal,
                                       schema=schema)
        self.article = makeContent(self.portal, 'PloneArticle', id='article')
        typeInfo = [ti for ti in listTypes(PROJECTNAME) if ti['name'] == 'SimpleSmartContent']
        dummy_out = StringIO()
        installTypes(self.portal, dummy_out, typeInfo, PROJECTNAME)



    def makeDummy(self):
        return self._dummy

    def testField_registered(self):
        self.failUnless(
            fieldDescriptionRegistry.get(
                'Products.PloneArticle.field.smartlist.SmartListField', None),
            "SmartListField is not registered by archetypes")

    def testAlwaysMultiValued(self):
        self.failUnless(SmartListField(multiValued=False).multiValued,
                        'Field does not enforce multivalued to True')

    def testAllowedTypes(self,):
        dummy = self.makeDummy()
        field = dummy.getField('smartlistfield')
        site_props = self.portal.portal_properties.site_properties
        types_not_searched = site_props.types_not_searched

        not_allowed_types = [t for t in field.getAllowedTypes(dummy)
                             if t in types_not_searched]
        self.failIf(not_allowed_types,
                    "Field allow to reference content normally excluded from search: %s"
                    % repr(not_allowed_types))

        restricted_field = dummy.getField('restricted_smartlistfield')
        self.assertEquals(restricted_field.getAllowedTypes(dummy),
                          ['PloneArticle',])

    def testAllowedCriterias(self):
        dummy = self.makeDummy()
        field = dummy.getField('restricted_smartlistfield')

        all_criterias = list(ALLOWED_CRITERIAS)
        all_criterias.sort(lowcase_cmp)
        self.assertEquals(field.getAllowedCriterias(dummy),
                          tuple(all_criterias))

        field = dummy.getField('smartlistfield')
        ct = getToolByName(self.portal, 'portal_catalog')

        self.assertEquals(field.getAllowedCriterias(dummy),
                          ALL_ALLOWED_CRITERIAS)

    def testGetSetSearchCriteria(self):
        empty_search = {'SearchableText': '', 'portal_types': (),}
        invalid_search = empty_search.copy()
        invalid_search.update({'invalid_index': False})

        dummy = self.makeDummy()
        field = dummy.getField('smartlistfield')

        # type checking
        self.assertRaises(TypeError, field.setSearchCriterias, dummy, None)

        # identity
        field.setSearchCriterias(dummy, empty_search)
        field_search = field.getSearchCriterias(dummy)
        self.failIf(empty_search != field_search)

        # invalid search keys must be discarded
        # we need to copy dict, mutator will change it
        field.setSearchCriterias(dummy, invalid_search.copy())
        field_search = field.getSearchCriterias(dummy)
        self.failIf(field_search == invalid_search)
        self.failIf(field_search != empty_search)

    def testGetSetAutoReference(self):
        dummy = self.makeDummy()
        field = dummy.getField('smartlistfield')

        self.assertEquals(field.getAutoReference(dummy), True)
        field.setAutoReference(dummy, False)
        self.assertEquals(field.getAutoReference(dummy), False)

        # test default parameter
        field = dummy.getField('restricted_smartlistfield')
        self.assertEquals(field.getAutoReference(dummy), False)

    def testExcludedUIDS(self):
        dummy = self.makeDummy()
        field = dummy.getField('smartlistfield')

        excluded_uids = field.getExcludedUIDs(dummy)
        self.assertEquals(excluded_uids, {},
                          "Initials excluded_uids not empty: '%s'"
                          % repr(excluded_uids))

        # verify it will accept only dict
        self.assertRaises(TypeError, field.setExcludedUIDs,
                          field, dummy, ())

        EXCLUDED = {'1': True}
        field.setExcludedUIDs(dummy, EXCLUDED)
        excluded_uids = field.getExcludedUIDs(dummy)
        self.assertEquals(excluded_uids, EXCLUDED)

    def testSet(self):

        content = makeContent(self.portal, "SimpleSmartContent",
                              id="smartlist_content")
        content_uid = content.UID()
        field = content.getField('smartlistfield')

        uct = getToolByName(self.portal, 'uid_catalog')
        ALL_UIDS = [b.UID for b in uct.searchResults() if b.UID != content_uid]
        total = len(ALL_UIDS)

        UIDS = ALL_UIDS[:-3]
        self.assert_(len(UIDS) >=3)
        REF_UIDS = UIDS[:2]
        REF_UIDS.sort()
        EXCLUDED = dict.fromkeys(UIDS[2:], True)

        field.set(content, REF_UIDS, uids_found=UIDS)
        value = field.getRaw(content)
        value.sort()
        self.assertEquals(value, REF_UIDS)

        excluded_uids = field.getExcludedUIDs(content)
        self.assertEquals(excluded_uids, EXCLUDED)

        # test with a common element:
        # in the new refs and old excluded:
        # => not in new excluded, present in new value
        #
        # and one in old value and 'found_uids' but not NEW_REFS::
        # not in new value, present in new excluded
        #
        NEW_UIDS = EXCLUDED.keys()[:1] + ALL_UIDS[-3:] + REF_UIDS[:1]
        NEW_REFS = NEW_UIDS[:2]
        ALL_REFS = REF_UIDS[1:] + NEW_REFS

        NEW_EXCLUDED = {NEW_UIDS[2]: True, NEW_UIDS[3]: True,
                        REF_UIDS[0]: True}

        ALL_EXCLUDED = dict(EXCLUDED)
        ALL_EXCLUDED.update(NEW_EXCLUDED)
        del ALL_EXCLUDED[NEW_REFS[0]]

        NEW_REFS.sort()
        ALL_REFS.sort()

        field.set(content, NEW_REFS, uids_found=NEW_UIDS)
        new_value = field.getRaw(content)
        new_value.sort()
        self.assertEquals(new_value, ALL_REFS)

        excluded_uids = field.getExcludedUIDs(content)
        self.assertEquals(excluded_uids, ALL_EXCLUDED)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(SmartListFieldTestCase))
    return suite



# -*- coding: utf-8 -*-
## Defines SmartListField
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Defines SmartListField
"""

__docformat__ = 'restructuredtext'

from types import DictType, ListType, TupleType

# Zope imports
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.PluginIndexes.interfaces import IUniqueValueIndex, IDateIndex, \
     IDateRangeIndex
from zope.interface import implements

# Archetypes imports
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Field import ReferenceField

from Products.ATContentTypes.criteria import _criterionRegistry
from Products.ATContentTypes.interface import IATTopicSearchCriterion
from Products.ATContentTypes.interface import IATTopicSortCriterion

#Products imports
from Products.PloneArticle.interfaces import ISmartListField
from Products.PloneArticle.widget import SmartListWidget

class SmartListField(ReferenceField):
    """
    A field that allow to reference content based on search criterias

    use 'allowed_types' to restrict selectable types
    """
    implements(ISmartListField)
    
    _properties = ReferenceField._properties.copy()
    _properties.update({
        'widget': SmartListWidget,
        'allowed_criterias': None, #catalog indexes allowed ids
        'default_criterias': (),
        'auto_reference': False,
        'results_len': 20, # number of results to show
        'sort_index': 'modified',
        'sort_order': 'reverse',
        })

    security  = ClassSecurityInfo()

    def __init__(self, name=None, **kwargs):
        """
        """
        ReferenceField.__init__(self, name, **kwargs)
        self.multiValued = True  # force multivalued
        self.search_criterias = {'SearchableText': '',
                                 'portal_types': (),
                                 }

    security.declarePublic('getAllowedTypes')
    def getAllowedTypes(self, instance):
        """Return the list of allowed portal_types for searching / referencing
        """
        pu = getToolByName(instance, 'plone_utils')
        allowed_types = list(self.allowed_types)
        return pu.getUserFriendlyTypes(allowed_types)


    security.declarePrivate('getAllowedCriterias')
    def getAllowedCriterias(self, instance):
        """return a tuple of allowed search criterias ids"""
        ct = getToolByName(instance, 'portal_catalog')
        criterias = self.allowed_criterias
        if criterias is None:
            criterias = ct.indexes()

        indexes = ct.getIndexObjects()
        criterias = [i.getId() for i in indexes
                     if i.getId() in criterias
                     and (i.getId() == 'SearchableText'
                          or (IUniqueValueIndex.providedBy(i)
                              and not (IDateIndex.providedBy(i) or
                                       IDateRangeIndex.providedBy(i))
                              )
                          )
                     ]
                     
        def lowcase_cmp(s1, s2):
            return cmp(s1.lower(), s2.lower())
        
        criterias.sort(lowcase_cmp)
        return tuple(criterias)

    security.declarePrivate('filterSearchCriterias')
    def filterSearchCriterias(self, instance, search_criterias):
        """
        Check params of search request, and remove invalids indexes
        """
        if type(search_criterias) is not DictType:
            raise TypeError, "Invalid type for search_criterias"

        allowed_criterias = self.getAllowedCriterias(instance)
        if allowed_criterias:
            for k in search_criterias.keys():
                if k not in allowed_criterias:
                    del search_criterias[k]

        # filter portal_type list
        # if portal_type is empty, fill it with searchable types
        pu = getToolByName(instance, 'plone_utils')
        searched_types = search_criterias.get('portal_type', [])
        searched_types = [t for t in searched_types if t != '']
        searched_types = pu.getUserFriendlyTypes(searched_types)

        #then intersect with field allowed_types
        allowed_types = self.getAllowedTypes(instance)
        search_criterias['portal_type'] = tuple([t for t in searched_types
                                                 if t in allowed_types])

        return search_criterias

    security.declarePublic('getAttrNameFor')
    def getAttrNameFor(self, propname):
        """
        Return the attribute name to look for on instance for storing property
        """
        return '%s_%s' % (self.getName(), propname)

    security.declarePublic('getSearchCriterias')
    def getSearchCriterias(self, instance):
        """
        Return a copy of current search_criterias dictionnary
        """
        search_criterias = getattr(instance.aq_explicit,
                                   self.getAttrNameFor('search_criterias'),
                                   self.search_criterias)
        return search_criterias.copy()

    security.declarePrivate('setSearchCriterias')
    def setSearchCriterias(self, instance, search_criterias):
        """
        adjust search_criterias dict to allowed queries and update field.
        This method has side-effect on 'search_criterias' variable.
        """
        search_criterias = self.filterSearchCriterias(instance, search_criterias)
        current = self.getSearchCriterias(instance)
        current.update(search_criterias)
        setattr(instance, self.getAttrNameFor('search_criterias'),
                self.filterSearchCriterias(instance, current))

    security.declarePublic('getExcludedUIDs')
    def getExcludedUIDs(self, instance):
        """
        Return the list of excluded uids
        """

        return getattr(aq_base(instance), self.getAttrNameFor('excluded_uids'),
                       {})

    security.declarePrivate('setExcludedUIDs')
    def setExcludedUIDs(self, instance, uids):
        """
        """
        if type(uids) is not DictType:
            raise TypeError, "expected a dictionnary, got %s" % repr(type(uids))
        
        setattr(instance, self.getAttrNameFor('excluded_uids'), uids)

    security.declarePublic('getAutoReference')
    def getAutoReference(self, instance):
        """
        """
        name = self.getAttrNameFor('auto_reference')
        return getattr(instance, name, self.auto_reference)

    security.declarePrivate('setAutoReference')
    def setAutoReference(self, instance, auto_ref):
        """
        """
        name = self.getAttrNameFor('auto_reference')
        setattr(instance, name, auto_ref)
        
    security.declarePublic('searchContents')
    def searchContents(self, instance, search_criterias=None):
        """
        return the 20 first brains sort on 'sort_index'
        """
        if search_criterias is None:
            search_criterias = self.getSearchCriterias(instance)
        else:
            if type(search_criterias) is not DictType:
                raise TypeError, "Invalid type for search_criterias: %s" % type(search_criterias)
            search_criterias = search_criterias.copy()

        search_criterias = self.filterSearchCriterias(instance, search_criterias)

        for k, v in search_criterias.items():
            if not v: #FIXME: break with bool search
                del search_criterias[k]

        search_criterias['sort_on'] = self.sort_index
        search_criterias['sort_order'] = self.sort_order
        search_criterias['sort_limit'] = self.results_len

        ct = getToolByName(instance, 'portal_catalog')
        return  ct.searchResults(**search_criterias)[:self.results_len]

    security.declarePrivate('get')
    def get(self, instance, aslist=False, **kwargs):
        """
        Return referenced objects + newer if auto_reference is set
        """
        result = ReferenceField.get(self, instance, aslist, **kwargs)

        if not self.getAutoReference(instance):
            return result

        uid_ct = getToolByName(instance, 'uid_catalog')
        x_uids = self.getExcludedUIDs(instance)
        # also exclude 'real' references to avoid doublons
        x_uids.update(dict.fromkeys([o.UID() for o in result]))
        
        # search with empty would return all catalog
        if len(x_uids):
            x_uids = uid_ct.searchResults(UID={'query': x_uids.keys(),})
            x_uids = {}.fromkeys([b.getPath() for b in x_uids], True)

        # brains path from uid catalog are relative to portal root
        urlTool = getToolByName(instance, 'portal_url')
        portal_path = '/'.join(urlTool.getPortalObject().getPhysicalPath())
        portal_path_len = len(portal_path) + 1 # trailling '/' counted

        auto_refs = self.searchContents(instance)
        auto_refs = [b.getObject() for b in auto_refs
                     if not x_uids.has_key(b.getPath()[portal_path_len:])]
        result.extend(auto_refs)
        return result
    
    security.declarePrivate('set')
    def set(self, instance, value, search_criterias=None,
            uids_found=None, ref_uids=(), auto_reference=None, **kwargs):
        """
        Mutator.

        value: list of uids, must be included in uids_found
        search_criterias': {...},
        uids_found': [...],  # list presented to the user
        'auto_reference': True or False
               }

        The from value and ref_uids the field builds a list of excluded uids and
        merge it with existing exclusion list. From this exclusion list, it
        removed already referenced uids (it means the user has unchecked them).

        Then it merge the old reference list with the new value, and from this
        new value it removes uids listed for exclusion (it means the user have
        checked them).

        To delete references, you must do a 'set' with empty value and the
        uids you wish to unref listed in 'uids_found'
        """

        if type(value) not in (ListType, TupleType):
            raise (TypeError,
                   "Invalid type for value: expected list or tuple, got '%s'"
                   % type(value))

        if search_criterias is not None:
            self.setSearchCriterias(instance, search_criterias)

        if auto_reference is not None:
            self.setAutoReference(instance, bool(auto_reference))

        old_value = dict.fromkeys(ReferenceField.getRaw(self, instance,))

        # update excluded_uids with uids listed but not checked for inclusion
        excluded_uids = self.getExcludedUIDs(instance)
        old_excluded_count = len(excluded_uids)
        update_excluded = False

        if uids_found is not None:
            new_excluded = dict.fromkeys([uid for uid in uids_found
                                          if uid not in value],
                                         True)
            update_excluded = len(new_excluded) != 0
            # remove old refs if they have been unchecked in widget
            for uid in new_excluded.iterkeys():
                if old_value.has_key(uid):
                    del old_value[uid]
            excluded_uids.update(new_excluded)
            
        # compute new value ensuring unique items in list
        value = dict.fromkeys(value)
        value.update(old_value)
        value = value.keys()

        # remove old excluded_uids if they have been checked in widget
        for k in excluded_uids.keys():
            if k in value:
                del excluded_uids[k]

        update_excluded = (update_excluded or
                           (len(excluded_uids) != old_excluded_count))

        # write excluded uids only if it has changed
        if update_excluded:
            self.setExcludedUIDs(instance, excluded_uids)
            
        ReferenceField.set(self, instance, value, **kwargs)
    
registerField(SmartListField,
              title="SmartListField",
              description=SmartListField.__doc__)



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: test_add_image.py 5836 2007-02-08 13:17:49Z roeder $
import unittest
from funittest import logical
from funittest import physical
from funittest import dataprovider
from funittest import scripts
from funittest import interpreter
from funittest.scenariorunner import Scenario
from funittest import Schema
from funittest import register_sce

import os

class AddImage(Scenario):
    schema = Schema()

    _uses_file_upload = 1

    def setUp(self):
        interpreter.open("")        
        user=dataprovider.cmfplone.user.get('sampleadmin')
        logical.cmfplone.application.login(user)
        self.article = dataprovider.plonearticle.article.get("article1")
        scripts.plonearticle.article.existing_article(self.article)

    def step_1(self):
        "Add an image to an article and save the article"
        interpreter.annotate("Test: Add an image to the article and save the article")
        image = dataprovider.plonearticle.articleimage.get("Image 1")
        # Normal attachment upload
        logical.plonearticle.article.add_image(image)
        logical.plonearticle.article.save_article(self.article)

    def test(self):
        self.expect_ok(1)

    def test_add_non_existing_image(self):
        "Try adding a non-existing image to an article"
        interpreter.annotate("Test: Try adding a non-existing image to an article")
        image = dataprovider.plonearticle.articleimage.get("Non-existing Image")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("images")
        physical.plonearticle.attachment_upload.access()
        physical.plonearticle.attachment_upload.fill(image)
        physical.plonearticle.attachment_upload.save(image, "error")        
        interpreter.verifyTextPresent("Image upload error: cannot identify image file")
        # Due to differences in the *chrome mode, the following error is not produced:
        # interpreter.verifyTextPresent("Error: Zero file size")
        physical.plonearticle.attachment_upload.cancel()  
        logical.plonearticle.article.save_article(self.article)
        
    def test_cancel_adding_an_image(self):
        "Cancel adding an image"
        interpreter.annotate("Test: Add an image to the article and cancel")
        image = dataprovider.plonearticle.articleimage.get("Image 1")
        physical.cmfplone.tab.access("edit")
        physical.cmfplone.schemata.access("images")
        physical.plonearticle.attachment_upload.access()
        physical.plonearticle.attachment_upload.fill(image)
        # Not saving here.
        physical.plonearticle.attachment_upload.cancel()  
        logical.plonearticle.article.save_article(self.article)

    def test_add_two_images_and_change_position(self):
        "Add two images and change their position"
        interpreter.annotate("Test: Add two images and change their position")

        images = []
        images.append(dataprovider.plonearticle.articleimage.get("Image 1"))
        images.append(dataprovider.plonearticle.articleimage.get("Image 2"))
        logical.plonearticle.article.add_multiple_images(images)

        # Change position of images       
        interpreter.click("//a[@class='file_move_down'][1]")
        interpreter.click("//a[@class='file_move_down'][1]")
        logical.plonearticle.article.save_article(self.article)
        
register_sce("PloneArticle", AddImage())



## Script (Python) "getRelPath"
##title=Helper method for ploneArticle Browser
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=path


portal=context.portal_url.getPortalObject()
portalPath=portal.getPhysicalPath()  
bPath=path.split('/')
lenPPath = len(portalPath)
relPath = '/'.join(bPath[lenPPath:])

return relPath



# -*- coding: utf-8 -*-
## PloneArticle
## A Plone document incorporating images, attachments and links, whith a free choice of layout.
## Copyright (C)2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# $Id: runalltests.py 5779 2007-01-21 17:41:41Z roeder $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py')) 
	
from funittest.texttestrunner import TextTestRunner
from funittest import load_model
load_model("PloneArticle", "plonearticle")

import test_add_file
import test_add_link
import test_add_image
import test_browse_image
import test_delete_file

def suite():
    return unittest.TestSuite((
        unittest.makeSuite(test_add_file.TestAddFile),
        unittest.makeSuite(test_add_link.TestAddLink),
        unittest.makeSuite(test_add_image.TestAddImage),
        unittest.makeSuite(test_browse_image.TestBrowseImage),
        unittest.makeSuite(test_delete_file.TestDeleteFile),
        ))
        
if __name__ == "__main__":
    result = TextTestRunner(verbosity=2).run(suite())
    sys.exit(not result.wasSuccessful())
